#! /bin/sh
#-------------------------------------------------------------------------------
# CompilOptions.sh
# Yvon Lafranche, Universite de Rennes 1.
#
#     Shell script for configuring compilation parameters.
#     A compiler is selected according to the operating system and the
#     associated compilation options are set.
#     These options were tested with the specified compiler.
#
#     Argument value and Return value
#     This script expects a single argument (see below) and returns the
#     corresponding information. The empty string is output if the algorithm
#     fails or the argument is unrecognized.
#
#           Argument value  |      Return value
#         ------------------|-----------------------------
#            "FC"           |  compiler name
#            "FFLAGS"       |  compiler options
#            "SEARCHEDFC"   |  list of searched compilers
#            other          |  empty string
#
#     Algorithm
#     A compiler is first picked among a list of system targeted compilers
#     and searched in the user's path. If this search fails, a last attempt
#     is carried on with the default compiler.
#
# Usage : sh CompilOptions.sh FC
#         sh CompilOptions.sh FFLAGS
#         sh CompilOptions.sh SEARCHEDFC
#-------------------------------------------------------------------------------
#
# Set the default compiler (that can be installed on most systems)
DFC="gfortran"
DFFLAGS="-W -fdefault-real-8 -O3"
#
# Determine whether the processor is Intel or not
if [ `uname -mp | grep 86 | wc -c` -gt 1 ]; then
  IntelProc="yes"; else IntelProc="no";
fi
#
# Test of expected compiler names.
# The first one found in the user's path is returned in FC.
# This variable is set to the empty string if no compiler is found.
testFCcmd () {
  FC=""
  for cmpName in $* ; do
    if [ `whichInPath $cmpName | wc -w` -eq 1 ]; then
      FC=$cmpName;
      break
    fi
  done
}
#
# Analogous to the Unix command which, except that aliases and shell
# built-in commands are ignored ; only the PATH environment variable is used.
whichInPath () {
  for dir in `echo $PATH | sed -e "s/:/ /"g` ; do
    if [ ! -d "$dir/$1" ] && [ -x "$dir/$1" ]; then
      printf "%s" $dir/$1
      break
    fi
  done
}
#
# Try to find a system targeted compiler or at least the default compiler
FFLAGS=""
OS=`uname`
case "$OS" in
#AIX-IBM =======================================================================
   AIX)
     SEARCHEDFC="f77 $DFC"
     testFCcmd $SEARCHEDFC
     case "$FC" in
  	f77)
#	f77 -V (xlf)
#	Not tested
  	       FFLAGS="-u -qrealsize=8 -O3" ;;
  	$DFC)
#       gfortran -fversion
#       
  	       FFLAGS=$DFFLAGS ;;
     esac
   ;;
#Darwin-MacOS X ================================================================
   Darwin)
     if [ $IntelProc = "yes" ]; then
       SEARCHEDFC="ifort $DFC"; else SEARCHEDFC="f77 $DFC";
     fi
     testFCcmd $SEARCHEDFC
     case "$FC" in
  	f77)
#	f77 -V (xlf)
#	FORTRAN 77 Compiler 8.2a
  	       FFLAGS="-u -qrealsize=8 -O3" ;;
  	ifort)
#       ifort -v
#       Version 10.1
  	       FFLAGS="-u -r8 -O3" ;;
  	$DFC)
#	gfortran -fversion
#	GNU Fortran 95 (GCC 4.1.0 20050911 (experimental))
  	       FFLAGS=$DFFLAGS ;;
     esac
   ;;
#Linux =========================================================================
   Linux)
     if [ $IntelProc = "yes" ]; then
       SEARCHEDFC="ifort $DFC"; else SEARCHEDFC="$DFC";
     fi
     testFCcmd $SEARCHEDFC
     case "$FC" in
  	ifort)
#       ifort -v
#       Version 10.1
  	       FFLAGS="-u -r8 -O3" ;;
  	$DFC)
#       gfortran -fversion
#       GNU Fortran 95 (GCC 4.0.0 20050519 (Red Hat 4.0.0-8))
#       GNU Fortran (GCC) 4.1.2 20070502 (Red Hat 4.1.2-12)   [Fedora Core 7]
  	       FFLAGS=$DFFLAGS ;;
     esac
   ;;
#SunOS =========================================================================
   SunOS)
     SEARCHEDFC="f77 $DFC"
     testFCcmd $SEARCHEDFC
     case "$FC" in
  	f77)
#	f77 -V
#	f90: Forte Developer 7 Fortran 95 7.0 2002/03/09
  	       FFLAGS="-xtypemap=real:64,double:64,integer:64 -O3" ;;
  	$DFC)
#       gfortran -fversion
#       
  	       FFLAGS=$DFFLAGS ;;
     esac
   ;;
#===============================================================================
esac
#
# Give a result according to what is requested
case $1 in
  FC)	       printf "%s" "$FC" ;;
  FFLAGS)      printf "%s" "$FFLAGS" ;;
  SEARCHEDFC)  printf "%s" "$SEARCHEDFC" ;;
  *)	       printf "" ;;
esac
#-------------------------------------------------------------------------------
