************************************************************************
*  Calcul d'exposants de singularites en elasticite anisotrope.
*  Yvon Lafranche, Universite de Rennes 1.
*  Voir documentation a l'adresse :
*        http://perso.univ-rennes1.fr/yvon.lafranche/exsiel/
************************************************************************
************************************************************************
* AFFECM : affectation d'une matrice dans une autre avec signe negatif
* AFFECP : affectation d'une matrice dans une autre avec signe positif
* ASSANU : assemblage de la matrice A(nu)
* BAESOL : calcul de la base de l'espace des solutions et de ses derivees
* BESANI : idem precedent dans le cas anisotrope
* BESISO : idem precedent dans le cas   isotrope
* CALFS  : calcul de la fonction singuliere associee a un nu
* CALKER : calcul du noyau de la matrice A(nu)
* CDLBOR : prise en compte des conditions aux limites en un bord
* CDLMIX : calcul des conditions aux limites de type mixte
* CDLNEU : calcul des conditions aux limites de type Neumann
* CDLNMG : calcul des C. L. de type Neumann (intermediaire)
* CDLN2D : calcul des C. L. de type Neumann (systeme 2D)
* CDLN3D : calcul des C. L. de type Neumann (systeme 3D)
* CGMBAS : changement de base
* COPRC  : copie (avec conversion de type) real <--> complex
* DETCAR : calcul du determinant caracteristique
* DPZSDZ : calcul de det'(A(z))/det(A(z))
* EPCTRL : ecriture des parametres de controle dans un fichier
* EVAFS  : evaluation de la fonction singuliere en un point
* KERANU : determ. de la dimension du noyau de A(nu) et impr. d'une base
* MACDET : determinant d'une matrice complexe (utilise LINPACK)
* MACINV : inverse d'une matrice complexe     (utilise LINPACK)
* TRACEP : calcul de la trace d'un produit de matrices
************************************************************************
      subroutine AFFECM(lmdx,md,nbl,nbc,pl,pc,lmax,ma)
      integer lmdx,nbl,nbc,pl,pc,lmax
      complex md(lmdx,nbc), ma(lmax,*)
*  Variables locales
      integer i,j, ja, plm1
*       ----------------------------------------------------------------
*       Affectation de la matrice -md (lignes 1 a nbl, colonnes 1 a nbc)
*       dans la zone de la matrice ma definie par les lignes pl a
*       pl+nbl-1 et les colonnes pc a pc+nbc-1.
*       Arguments d'entree  : lmdx,md,nbl,nbc,pl,pc,lmax
*       Arguments de sortie : ma
*       ----------------------------------------------------------------
      plm1 = pl - 1
      ja = pc - 1
      do 10 j=1,nbc
       ja = ja + 1
       do 10 i=1,nbl
   10   ma(i+plm1,ja) = -md(i,j)
      end
************************************************************************
      subroutine AFFECP(lmdx,md,nbl,nbc,pl,pc,lmax,ma)
      integer lmdx,nbl,nbc,pl,pc,lmax
      complex md(lmdx,nbc), ma(lmax,*)
*  Variables locales
      integer i,j, ja, plm1
*       ---------------------------------------------------------------
*       Affectation de la matrice md (lignes 1 a nbl, colonnes 1 a nbc)
*       dans la zone de la matrice ma definie par les lignes pl a
*       pl+nbl-1 et les colonnes pc a pc+nbc-1.
*       Arguments d'entree  : lmdx,md,nbl,nbc,pl,pc,lmax
*       Arguments de sortie : ma
*       ---------------------------------------------------------------
      plm1 = pl - 1
      ja = pc - 1
      do 10 j=1,nbc
       ja = ja + 1
       do 10 i=1,nbl
   10   ma(i+plm1,ja) = md(i,j)
      end
************************************************************************
      subroutine ASSANU(cdlim,dmeso,dim,nbcxx,nbdom,dimanu,omega,contx,
     & cc,alpha,qplus,phi,dphidt,cneu,dphidn,ddpdtn,cneudn,nu,anu,anudn)
      integer cdlim(2),dmeso, dim, nbcxx, nbdom, dimanu, contx(nbdom)
      real omega(0:nbdom), cc(nbcxx,nbdom)
      complex alpha(dim,nbdom), qplus(dim,dim,nbdom)
      complex phi(dim,dmeso), dphidt(dim,dmeso), cneu(dim,dmeso)
      complex dphidn(dim,dmeso), ddpdtn(dim,dmeso),cneudn(dim,dmeso)
      complex nu,anu(dimanu,dimanu),anudn(dimanu,dimanu)
*  Variables locales
      integer i,j, ia,ja,it, nbor
      real theta, nor(2)
      complex zero
*  Initialisation
      data zero /(0.,0.)/
*       ------------------------------------------------------------
*       Assemblage de la matrice A(nu) avec prise en compte des
*       conditions aux limites.
*       Arguments d'entree  : cdlim,dmeso,dim,nbcxx,nbdom,
*                             dimanu,omega,contx,cc,alpha,qplus,nu
*       Arguments de sortie : anu,anudn
*       Tableaux de travail : phi,dphidt,cneu,dphidn,ddpdtn,cneudn
*        cdlim  : conditions aux limites en omega(0) et omega(nbdom)
*        dmeso  : dimension de l'espace des solutions
*        dim    : dimension de l'espace
*        nbcxx  : nombre max de coefficients (1ere dimension de cc)
*        nbdom  : nombre de domaines
*        dimanu : 1ere dimension de la matrice anu
*        omega  : angles definissant la geometrie des domaines
*        contx  : contexte de calcul pour chaque domaine
*        cc     : coefficients de la matrice de rigidite pour chaque
*                 domaine
*        alpha  : racines du determinant du symbole M+
*        qplus  : vecteurs directeurs de la base correspondant a M+
*        phi    : vecteurs de la base de l'espace des solutions
*                 (phi(.,i), ieme vecteur, i=1,2*dim)
*        dphidt : derivee par rapport a theta des vecteurs de base
*                 (dphidt(.,i), ieme vecteur, i=1,2*dim)
*        cneu   : stockage des conditions de Neumann
*        dphidn : derivee par rapport a nu des vecteurs de base
*                 (dphidn(.,i), ieme vecteur, i=1,2*dim)
*        ddpdtn : derivee par rapport a theta et a nu des vecteurs
*                 de base (ddpdtn(.,i), ieme vecteur, i=1,2*dim)
*        cneudn : derivee par rapport a nu des conditions de Neumann
*                 (cneudn(.,i), ieme vecteur, i=1,2*dim)
*        nu     : exposant de singularite
*        anu    : matrice des conditions aux limites et des
*                 conditions de transmission
*        anudn  : derivee de la matrice anu par rapport a nu
*       ------------------------------------------------------------
*       Initialisation de anu et anudn
      do 10 j=1,dimanu
       do 10 i=1,dimanu
        anu  (i,j) = zero
   10   anudn(i,j) = zero
*
*       Condition limite en omega(0) (domaine No 1, bord No 1)
      i = 1
      ia = 1
      ja = 1
      it = ia + dim + (nbdom-1)*dmeso
      theta = omega(0)
      nor(1) = -sin(theta)
      nor(2) =  cos(theta)
      nbor = 1
      call BAESOL(dim,theta,nu,contx(i),cc(1,i),alpha(1,i),qplus(1,1,i),
     & phi,dphidt,dphidn,ddpdtn)
*       Conditions de Neumann inutiles dans le cas Dirichlet
      if (cdlim(nbor).gt. 1) call CDLNEU(cc(1,i),nor,nor,dim,dmeso,phi,
     & dphidt,dphidn,ddpdtn,nu,cneu,cneudn)
      call CDLBOR(cdlim(nbor),nbor,ia,ja,it,nor,dim,dmeso,dimanu,
     & phi,cneu,anu)
      call CDLBOR(cdlim(nbor),nbor,ia,ja,it,nor,dim,dmeso,dimanu,
     & dphidn,cneudn,anudn)
*
*       Conditions de transmission entre les domaines (le domaine i
*       est compris entre omega(i-1) et omega(i))
      ia = ia + dim - dmeso
      ja = ja - dmeso
      do 20 i=1,nbdom-1
       ia = ia + dmeso
       ja = ja + dmeso
       theta = omega(i)
       nor(1) = -sin(theta)
       nor(2) =  cos(theta)
*        Condition de Dirichlet en omega(i) domaine i
       call BAESOL(dim,theta,nu,contx(i),cc(1,i),alpha(1,i),qplus(1,1,i)
     &  ,phi,dphidt,dphidn,ddpdtn)
       call AFFECP(dim,phi   ,dim,dmeso,ia,ja,dimanu,anu)
       call AFFECP(dim,dphidn,dim,dmeso,ia,ja,dimanu,anudn)
*        Condition de Neumann en omega(i) domaine i
       call CDLNEU(cc(1,i),nor,nor,dim,dmeso,phi,dphidt,dphidn,ddpdtn,
     &  nu,cneu,cneudn)
       call AFFECP(dim,cneu  ,dim,dmeso,ia+dim,ja,dimanu,anu)
       call AFFECP(dim,cneudn,dim,dmeso,ia+dim,ja,dimanu,anudn)
*
*        Condition de Dirichlet en omega(i) domaine i+1
       j = i + 1
       call BAESOL(dim,theta,nu,contx(j),cc(1,j),alpha(1,j),qplus(1,1,j)
     &  ,phi,dphidt,dphidn,ddpdtn)
       call AFFECM(dim,phi   ,dim,dmeso,ia,ja+dmeso,dimanu,anu)
       call AFFECM(dim,dphidn,dim,dmeso,ia,ja+dmeso,dimanu,anudn)
*        Condition de Neumann en omega(i) domaine i+1
       call CDLNEU(cc(1,j),nor,nor,dim,dmeso,phi,dphidt,dphidn,ddpdtn,
     &  nu,cneu,cneudn)
       call AFFECM(dim,cneu  ,dim,dmeso,ia+dim,ja+dmeso,dimanu,anu)
       call AFFECM(dim,cneudn,dim,dmeso,ia+dim,ja+dmeso,dimanu,anudn)
   20 continue  
*
*       Condition limite en omega(n) (domaine No n, bord No 2)
      i = nbdom
      ia = ia + dmeso
      ja = ja + dmeso
      it = 1
      theta = omega(nbdom)
      nor(1) = -sin(theta)
      nor(2) =  cos(theta)
      nbor = 2
      call BAESOL(dim,theta,nu,contx(i),cc(1,i),alpha(1,i),qplus(1,1,i),
     & phi,dphidt,dphidn,ddpdtn)
c MODIF
c      write(*,*) 'THETA = ', theta
c      call IMPTAB(' Matrice qplus :',qplus(1,1,i),dim,dim,dim)
c      call IMPTAB(' Matrice phi :',phi,dim,dim,dmeso)
c      call IMPTAB(' Matrice dphidt :',dphidt,dim,dim,dmeso)
*       Conditions de Neumann inutiles dans le cas Dirichlet
      if (cdlim(nbor).gt. 1) call CDLNEU(cc(1,i),nor,nor,dim,dmeso,phi,
     & dphidt,dphidn,ddpdtn,nu,cneu,cneudn)
      call CDLBOR(cdlim(nbor),nbor,ia,ja,it,nor,dim,dmeso,dimanu,
     & phi,cneu,anu)
      call CDLBOR(cdlim(nbor),nbor,ia,ja,it,nor,dim,dmeso,dimanu,
     & dphidn,cneudn,anudn)
      end
************************************************************************
      subroutine BAESOL(dim,theta,nu,contx,cc,alpha,qplus,phi,dphidt,
     & dphidn,ddpdtn)
      integer dim, contx
      real theta, cc(*)
      complex nu, alpha(dim), qplus(dim,dim), phi(dim,*)
      complex dphidt(dim,*), dphidn(dim,*), ddpdtn(dim,*)
*       ------------------------------------------------------------
*       Pour un domaine donne, calcul de la base de l'espace des
*       solutions et de ses derivees par rapport a nu et theta.
*
*       Arguments d'entree  : dim,theta,nu,contx,cc,alpha,qplus
*       Arguments de sortie : phi,dphidt,dphidn,ddpdtn
*        dim    : dimension de l'espace
*        theta  : angle relatif au domaine considere
*        nu     : exposant de singularite
*        contx  : contexte de calcul du domaine considere
*        cc     : coefficients de la matrice de rigidite du
*                 domaine considere
*        alpha  : racines du determinant du symbole M+
*        qplus  : vecteurs directeurs de la base correspondant a M+
*        phi    : vecteurs de la base de l'espace des solutions
*                 (phi(.,i), ieme vecteur, i=1,2*dim)
*        dphidt : derivee par rapport a theta des vecteurs de base
*                 (dphidt(.,i), ieme vecteur, i=1,2*dim)
*        dphidn : derivee par rapport a nu des vecteurs de base
*                 (dphidn(.,i), ieme vecteur, i=1,2*dim)
*        ddpdtn : derivee par rapport a theta et a nu des vecteurs
*                 de base (ddpdtn(.,i), ieme vecteur, i=1,2*dim)
*       ------------------------------------------------------------
      if (contx .eq. 1) then
*
*        Cas isotrope
       call BESISO(dim,theta,nu,cc,phi,dphidt,dphidn,ddpdtn)
      else
*
*        Cas anisotrope
       call BESANI(dim,theta,nu,alpha,qplus,phi,dphidt,dphidn,ddpdtn)
      endif
      end
************************************************************************
      subroutine BESANI(dim,theta,nu,alpha,qplus,phi,dphidt,dphidn,
     & ddpdtn)
      integer dim
      real theta
      complex nu, alpha(dim), qplus(dim,dim), phi(dim,*)
      complex dphidt(dim,*), dphidn(dim,*), ddpdtn(dim,*)
*  Variables locales
      integer i,j,jpdim
      complex icomp, un, epitn,emitn, e2it
      complex beta,gam,gamp1,delta,deltab,logam, fp,fm, num1
*       ------------------------------------------------------------
*       Pour un domaine donne, calcul de la base de l'espace des
*       solutions et de ses derivees par rapport a nu et theta, dans
*       le cas ou le materiau est anisotrope.
*
*       Arguments d'entree  : dim,theta,nu,alpha,qplus
*       Arguments de sortie : phi,dphidt,dphidn,ddpdtn
*        dim    : dimension de l'espace
*        theta  : angle relatif au domaine considere
*        nu     : exposant de singularite
*        alpha  : racines du determinant du symbole M+
*        qplus  : vecteurs directeurs de la base correspondant a M+
*        phi    : vecteurs de la base de l'espace des solutions
*                 (phi(.,i), ieme vecteur, i=1,2*dim)
*        dphidt : derivee par rapport a theta des vecteurs de base
*                 (dphidt(.,i), ieme vecteur, i=1,2*dim)
*        dphidn : derivee par rapport a nu des vecteurs de base
*                 (dphidn(.,i), ieme vecteur, i=1,2*dim)
*        ddpdtn : derivee par rapport a theta et a nu des vecteurs
*                 de base (ddpdtn(.,i), ieme vecteur, i=1,2*dim)
*       ------------------------------------------------------------
      icomp = cmplx(0.,1.)
      un = cmplx(1.)
      epitn = exp(cmplx(0.,theta) * nu)
      emitn = un / epitn
      e2it = exp(cmplx(0.,2.*theta))
      num1 = nu - un
      do 30 j=1,dim
       jpdim = j+dim
       gam = alpha(j) * e2it
       gamp1 = gam + un
       logam = log(gamp1)
       delta = logam - cmplx(0.,theta)
       deltab = conjg(delta)
       fp = emitn * exp(num1 * logam)
       fm = epitn * exp(num1 * conjg(logam))
*       Base et derivee par rapport a nu
       do 10 i=1,dim
        beta = gamp1 * qplus(i,j)
        phi(i,j)     = fp * beta
        phi(i,jpdim) = fm * conjg(beta)
        dphidn(i,j)     = delta  * phi(i,j)
        dphidn(i,jpdim) = deltab * phi(i,jpdim)
   10 continue
*       Derivee par rapport a theta et
*       derivee seconde par rapport a theta et a nu
       beta = icomp * (gam - un)
       do 20 i=1,dim
        gam = beta * qplus (i,j)
        gamp1 = fp * gam
        dphidt(i,j) = gamp1 * nu
        ddpdtn(i,j) = gamp1 * (un + nu*delta)
        gamp1 = fm * conjg(gam)
        dphidt(i,jpdim) = gamp1 * nu
        ddpdtn(i,jpdim) = gamp1 * (un + nu*deltab)
   20 continue
   30 continue
*        Changement de base, motive par le fait que si l'exposant
*        de singularite est reel, alors la valeur de la fonction
*        singuliere est aussi reelle.
*        De plus, la matrice A(nu) comporte davantage d'elements
*        nuls dans la nouvelle base.
      call CGMBAS(dim,phi)
      call CGMBAS(dim,dphidt)
      call CGMBAS(dim,dphidn)
      call CGMBAS(dim,ddpdtn)
      end
************************************************************************
      subroutine BESISO(dim,theta,nu,cc,phi,dphidt,dphidn,ddpdtn)
      integer dim
      real theta, cc(*)
      complex nu, phi(dim,*)
      complex dphidt(dim,*), dphidn(dim,*), ddpdtn(dim,*)
*  Variables locales
      integer i,j, cmu(2:3)
      real lpm, lp3m
      complex psi(2), val
*  Initialisation
      data cmu /6, 21/
*       ------------------------------------------------------------
*       Pour un domaine donne, calcul de la base de l'espace des
*       solutions et de ses derivees par rapport a nu et theta, dans
*       le cas ou le materiau est isotrope.
*
*       Arguments d'entree  : dim,theta,nu,cc
*       Arguments de sortie : phi,dphidt,dphidn,ddpdtn
*        dim    : dimension de l'espace
*        theta  : angle relatif au domaine considere
*        nu     : exposant de singularite
*        cc     : coefficients de la matrice de rigidite du
*                 domaine considere
*        phi    : vecteurs de la base de l'espace des solutions
*                 (phi(.,i), ieme vecteur, i=1,2*dim)
*        dphidt : derivee par rapport a theta des vecteurs de base
*                 (dphidt(.,i), ieme vecteur, i=1,2*dim)
*        dphidn : derivee par rapport a nu des vecteurs de base
*                 (dphidn(.,i), ieme vecteur, i=1,2*dim)
*        ddpdtn : derivee par rapport a theta et a nu des vecteurs
*                 de base (ddpdtn(.,i), ieme vecteur, i=1,2*dim)
*       ------------------------------------------------------------
      i = cmu(dim)
      lpm  = cc(2) + cc(i)
      lp3m = cc(1) + cc(i)
*
      val = cmplx(theta) * (nu-cmplx(2.))
      psi(1) = lpm * sin(val)
      psi(2) = lpm * cos(val)
*
      val = cmplx(theta) * nu
      phi(1,1) =  cos(val)
      phi(2,1) = -sin(val)
      phi(1,2) = -phi(2,1)
      phi(2,2) =  phi(1,1)
      phi(1,3) =  lp3m * phi(1,1)
      phi(2,3) = -lp3m * phi(2,1)
      val = nu * psi(1)
      phi(1,4) = -phi(2,3) + val
      phi(2,3) =  phi(2,3) + val
      val = nu * psi(2)
      phi(2,4) = phi(1,3) + val
      phi(1,3) = phi(1,3) - val
*
      dphidt(1,1) = -nu * phi(1,2)
      dphidt(2,1) = -nu * phi(2,2)
      dphidt(1,2) =  nu * phi(1,1)
      dphidt(2,2) =  nu * phi(2,1)
      dphidt(1,3) = nu * ( phi(1,4) - 2*psi(1))
      dphidt(2,3) = nu * ( phi(2,4) - 2*psi(2))
      dphidt(1,4) = nu * (-phi(1,3) - 2*psi(2))
      dphidt(2,4) = nu * (-phi(2,3) + 2*psi(1))
*
      dphidn(1,1) = -cmplx(theta) * phi(1,2)
      dphidn(2,1) = -cmplx(theta) * phi(2,2)
      dphidn(1,2) =  cmplx(theta) * phi(1,1)
      dphidn(2,2) =  cmplx(theta) * phi(2,1)
      dphidn(1,3) =  cmplx(theta) * phi(1,4) - psi(2)
      dphidn(2,3) =  cmplx(theta) * phi(2,4) + psi(1)
      dphidn(1,4) = -cmplx(theta) * phi(1,3) + psi(1)
      dphidn(2,4) = -cmplx(theta) * phi(2,3) + psi(2)
*
      ddpdtn(1,1) = -phi(1,2) - nu*dphidn(1,2)
      ddpdtn(2,1) = -phi(2,2) - nu*dphidn(2,2)
      ddpdtn(1,2) =  phi(1,1) + nu*dphidn(1,1)
      ddpdtn(2,2) =  phi(2,1) + nu*dphidn(2,1)
      val = nu-cmplx(2.)
      ddpdtn(1,3) =  phi(1,4) + cmplx(theta)*dphidt(1,4) + val*psi(1)
      ddpdtn(2,3) =  phi(2,4) + cmplx(theta)*dphidt(2,4) + val*psi(2)
      ddpdtn(1,4) = -phi(1,3) - cmplx(theta)*dphidt(1,3) + val*psi(2)
      ddpdtn(2,4) = -phi(2,3) - cmplx(theta)*dphidt(2,3) - val*psi(1)
*
      if (dim .eq. 3) then
       do 10 i=1,4
        phi(3,i) = 0.
        dphidt(3,i) = 0.
        dphidn(3,i) = 0.
   10   ddpdtn(3,i) = 0.
       do 20 i=1,2
        do 20 j=5,6
         phi(i,j) = 0.
         dphidt(i,j) = 0.
         dphidn(i,j) = 0.
   20    ddpdtn(i,j) = 0.
       phi(3,5) = phi(1,2)
       phi(3,6) = phi(2,2)
       dphidt(3,5) =  nu * phi(3,6)
       dphidt(3,6) = -nu * phi(3,5)
       dphidn(3,5) =  cmplx(theta) * phi(3,6)
       dphidn(3,6) = -cmplx(theta) * phi(3,5)
       val = cmplx(theta) * nu
       ddpdtn(3,5) =  phi(3,6) + nu*dphidn(3,6)
       ddpdtn(3,6) = -phi(3,5) - nu*dphidn(3,5)
      endif
      end
************************************************************************
      subroutine CALFS(unlo,dim,nbcxx,nbdom,dmeso,dimanu,sseprc,parcou,
     & nu,contx,cc,coef,dimker,omega,alpha,qplus,phi,dphidt,dphidn,
     & ddpdtn,cneu,cneudn)
      integer unlo(*),dim,nbcxx,nbdom,dmeso,dimanu,dimker, contx(nbdom)
      complex nu,coef(dimanu,dimanu)
      real sseprc,parcou, cc(nbcxx,nbdom), omega(0:nbdom)
      complex alpha(dim,nbdom), qplus(dim,dim,nbdom)
      complex phi(dim,dmeso), dphidt(dim,dmeso), cneu(dim,dmeso)
      complex dphidn(dim,dmeso), ddpdtn(dim,dmeso),cneudn(dim,dmeso)
*  Variables locales
      integer codret,iusgf,iutrf,ium
      integer i,j,k,l,m,nbp,nbpx,nbpts
      integer typi,typr,typc, ndec(0:2), tbi
      parameter (nbpts = 200, typi=0, typr=1, typc=2)
      real theta, h, fact, nor(2), ncb(2), tbr
      complex tbc
      character nomrep*160, nomfic*30, ext*9
      integer lgrep,lgfic,lgext
*  Initialisation
      data ndec /11, 15, 15/ 
*       ----------------------------------------------------------------
*       Calcul de la (ou les) fonction(s) singuliere(s) angulaire(s) et
*       des contraintes associees a l'exposant de singularite nu.
*       Les valeurs de chaque fonction singuliere sont imprimees dans le
*       fichier associe a l'unite logique unlo(3) precedees des
*       informations destinees a leur exploitation.
*       Les valeurs des contraintes sont calculees en meme temps et sont
*       imprimees dans le fichier associe a l'unite logique unlo(5)
*       (contraintes tangentielles).
*       Ensuite, un fichier de commande Matlab (M-file) est cree.
*       Execute dans l'environnement Matlab, il permet aussi d'obtenir
*       les courbes des fonctions singulieres, et eventuellement
*       d'effectuer des calculs annexes.
*       Arguments d'entree  : unlo,dim,nbcxx,nbdom,dmeso,dimanu,sseprc,
*                             parcou,nu,contx,cc,coef,dimker,omega,
*                             alpha,qplus
*       Tableaux de travail : phi,dphidt,dphidn,ddpdtn,cneu,cneudn
*        unlo   : numeros des unites logiques des fichiers ou
*                 sont imprimes les resultats
*        dim    : dimension de l'espace
*        nbcxx  : nombre max de coefficients (1ere dimension de cc)
*        nbdom  : nombre de domaines
*        dmeso  : dimension de l'espace des solutions
*        dimanu : 1ere dimension de la matrice anu
*        sseprc : seuil de separation des racines reelles et complexes
*        parcou : parametre courant pilotant le calcul
*        nu     : exposant de singularite
*        contx  : contexte de calcul pour chaque domaine
*        cc     : coefficients de la matrice de rigidite pour chaque
*                 domaine
*        coef   : coefficients de la combinaison lineaire definissant
*                 la fonction singuliere (noyau de A(nu))
*        dimker : dimension du noyau de A(nu)
*        omega  : angles definissant la geometrie des domaines
*        alpha  : racines du determinant du symbole M+
*        qplus  : vecteurs directeurs de la base correspondant a M+
*        phi    : vecteurs de la base de l'espace des solutions
*                 (phi(.,i), ieme vecteur, i=1,2*dim)
*        dphidt : derivee par rapport a theta des vecteurs de base
*                 (dphidt(.,i), ieme vecteur, i=1,2*dim)
*        dphidn : derivee par rapport a nu des vecteurs de base
*                 (dphidn(.,i), ieme vecteur, i=1,2*dim)
*        ddpdtn : derivee par rapport a theta et a nu des vecteurs
*                 de base (ddpdtn(.,i), ieme vecteur, i=1,2*dim)
*        cneu   : stockage des conditions de Neumann
*        cneudn : derivee par rapport a nu des conditions de Neumann
*                 (cneudn(.,i), ieme vecteur, i=1,2*dim)
*       ----------------------------------------------------------------
      call KELNOM(unlo(3),nomrep,nomfic,ext,lgrep,lgfic,lgext)
*      dcnb = index(nomfic,' ')-1
*      if (dcnb .eq. 0) dcnb = len(nomfic)
*
*       Creation des fichiers de valeurs
*
*        1) Partie "controle"
*
      fact = nbpts / (omega(nbdom)-omega(0))
*
*          Fonctions singulieres
      iusgf  = unlo(3)
      call EPCTRL(iusgf,dim,nbdom,dimker,sseprc,parcou,fact,omega,nu,
     &nomfic,lgfic,codret)
      if (codret.ne.0) goto 60
*
*          Contraintes
*          1. tangentielles (sTRess Fonction)
      iutrf  = unlo(5)
      call EPCTRL(iutrf,dim,nbdom,dimker,sseprc,parcou,fact,omega,nu,
     &nomfic,lgfic,codret)
      if (codret.ne.0) goto 60
*
*        2) Partie "valeurs"
*
      do 40 m=1,dimker
*       Calcul pour la m-ieme fonction singuliere
       k = dimanu-dimker+m
       j = 1
       do 30 i=1,nbdom
        h = omega(i)-omega(i-1)
        nbpx = nint(h*fact)
        h = h / (nbpx-1)
        do 20 nbp=0,nbpx-1
         theta = omega(i-1) + nbp*h
         call BAESOL(dim,theta,nu,contx(i),cc(1,i),alpha(1,i),
     &    qplus(1,1,i),phi,dphidt,dphidn,ddpdtn)
*
*          Fonctions singulieres
         call EVAFS(dim,phi,coef(j,k),cneudn)
         write(iusgf,50,err=60) theta, (cneudn(l,1),l=1,dim)
*
*          Contraintes sur un cercle de rayon 1 centre au coin.
*          1. Traction tangentielle, donc normale au bord des domaines
         ncb(1) = -sin(theta)
         ncb(2) =  cos(theta)
         call CDLNEU(cc(1,i),ncb,ncb,dim,dmeso,phi,
     &    dphidt,dphidn,ddpdtn,nu,cneu,cneudn)
         call EVAFS(dim,cneu,coef(j,k),cneudn)
         write(iutrf,50,err=60) theta, (cneudn(l,1),l=1,dim)
   20   continue
   30  j = j + dmeso
   40 continue
*
*       Creation du M-file
*
      ium  = unlo(4)
      call IMPMES(ium,'CALFS',-1,' ',codret)
      if (codret .ne. 0) goto 70
      call CRTBMT(ium,ndec(typi),'contx',typi,nbdom,1,1,contx,tbr,tbc,
     & codret)
      if (codret .ne. 0) goto 70
      call CRTBMT(ium,ndec(typr),'omega',typr,nbdom+1,1,1,tbi,omega,tbc,
     & codret)
      if (codret .ne. 0) goto 70
      call CRTBMT(ium,ndec(typc),'alpha',typc,dim,nbdom,1,tbi,tbr,alpha,
     & codret)
      if (codret .ne. 0) goto 70
      call CRTBMT(ium,ndec(typc),'qplus',typc,dim,dim,nbdom,tbi,tbr,
     & qplus,codret)
      if (codret .ne. 0) goto 70
      k = dimanu-dimker+1
      call CRTBMT(ium,ndec(typc),'coef',typc,dimanu,dimker,1,tbi,tbr,
     & coef(1,k),codret)
      if (codret .ne. 0) goto 70
      call CRTBMT(ium,ndec(typc),'nu',typc,1,1,1,tbi,tbr,nu,codret)
      if (codret .ne. 0) goto 70
      write(ium,*,err=70) 'nbpts = ', nbpts, ';'
      write(ium,*,err=70) 'sseprc = ', sseprc, ';'
      write(ium,*,err=70) 'parcou = ', parcou, ';'
      write(ium,*,err=70) 'repfs(contx,omega,nu,alpha,qplus,coef,nbpts,'
     & ,'sseprc,parcou,''',nomfic(:lgfic),''',''',ext(:lgext),''');'
      return
*
   50 format(g23.15e3,6(1x,g23.15e3))
*
   60 call IMPMES(0,'CALFS',2,' ',codret)
      return
   70 call IMPMES(0,'CALFS',3,' ',codret)
      end
************************************************************************
      subroutine CALKER(unlo,nu,multi,action,parcou,codret)
      integer unlo(*),multi,action(2),codret
      complex nu
      real parcou
*  Communs
      include 'singul.inc'
*  Variables locales
      integer info, dimker
      real rsvd(dmesox*nbdomx,2)
*       -----------------------------------------------------
*       Calcul du noyau de la matrice A(nu).
*       Arguments d'entree  : unlo,nu,multi,action,parcou
*        unlo   : numeros des unites logiques des fichiers ou
*                 sont imprimes les resultats
*        nu     : exposant de singularite
*        multi  : multiplicite de nu
*        action : caracterise l'action a realiser
*        parcou : parametre courant pilotant le calcul
*        codret : code de retour egal a 0 si RAS, #0 sinon.
*       -----------------------------------------------------
      call IMPMES(  0,'CALKER',-1,' ',codret)
      write(*,'(''nu = '',g23.15e3,1x,sp,g23.15e3,'' *i'')') nu
*        Assemblage de la matrice A(nu)
      call ASSANU(cdlim,dmeso,dim,nbcxx,nbdom,dimanu,omega,contx,
     & cc,alpha,qplus,phi,dphidt,cneu,dphidn,ddpdtn,cneudn,nu,anu,anudn)
*                                                 _T
*        Decomposition SVD de A(nu) : A(nu) = U S V  (sans le calcul de U)
      if (abs(aimag(nu)) .gt. sseprc) then
*       nu est complexe
       call CSVDC(anu,dimanu,dimanu,dimanu,anudn,anudn(1,2),anu,dimanu,
     &  anu,dimanu,work,1,info)
      else
*       nu est reel
       call COPRC(ranu,anu,dimanu,dimanu,-1)
       call SSVDC(ranu,dimanu,dimanu,dimanu,rsvd,rsvd(1,2),ranu,dimanu,
     &  ranu,dimanu,rwork,1,info)
       call COPRC(ranu,anu,dimanu,dimanu,1)
       call COPRC(rsvd,anudn,dimanu,1,1)
      endif
*
      call KERANU(unlo(1),nu,multi,anu,dimanu,anudn,info,dimker,codret)
*
*       Calcul de la (des) fonction(s) singuliere(s) et des contraintes
*       associee(s) au noyau precedent
      if (action(2) .ge. 2) then
       if (codret .eq. 0) then
        call CALFS(unlo,dim,nbcxx,nbdom,dmeso,dimanu,sseprc,parcou,nu,
     &   contx,cc,anu,dimker,omega,alpha,qplus,phi,dphidt,dphidn,
     &   ddpdtn,cneu,cneudn)
       else
        call IMPMES(      0,'CALKER',2,' ',codret)
        call IMPMES(unlo(1),'CALKER',2,' ',codret)
       endif
      endif
      end
************************************************************************
      subroutine CDLBOR(tclim,nbor,ia,ja,it,nor,dim,dmeso,dimanu,
     & phi,cneu,anu)
      integer tclim,nbor,ia,ja,it,dim,dmeso,dimanu
      real nor(2)
      complex phi(dim,dmeso), cneu(dim,dmeso), anu(dimanu,dimanu)
*       ---------------------------------------------------------------
*       Prise en compte des conditions aux limites au bord nbor
*       de l'ensemble des domaines.
*       Arguments d'entree  : tclim,nbor,ia,ja,it,nor,dim,
*                             dmeso,dimanu,phi,cneu
*       Arguments de sortie : anu
*        tclim  : type de condition aux limites sur le bord considere
*        nbor   : numero du bord de la reunion des domaines
*                 omega(0) (resp. (n)) correspond a nbor = 1 (resp. 2)
*        ia,ja  : adresse de stockage des coefficients dans anu
*        it     : idem, mais n'est utilise que dans le cas transmission
*        nor    : normale a la frontiere du domaine considere
*        dim    : dimension de l'espace
*        dmeso  : dimension de l'espace des solutions
*        dimanu : 1ere dimension de la matrice anu
*        phi    : vecteurs de la base de l'espace des solutions
*                 ou derivee par rapport a nu des vecteurs de base
*                 (phi(.,i), ieme vecteur, i=1,2*dim)
*        cneu   : stockage des conditions de Neumann ou
*                 derivee par rapport a nu des conditions de Neumann
*                 (cneu(.,i), ieme vecteur, i=1,2*dim)
*        anu    : matrice des conditions aux limites et des
*                 conditions de transmission
*                 ou derivee de la matrice anu par rapport a nu
*       ---------------------------------------------------------------
*
      goto (10,20,30,30,50) tclim
*
*       Dirichlet
   10 call AFFECP(dim,phi,dim,dmeso,ia,ja,dimanu,anu)
      goto 99
*
*       Neumann
   20 call AFFECP(dim,cneu,dim,dmeso,ia,ja,dimanu,anu)
      goto 99
*
*       Mixte Dirichlet normal     - Neumann tangentiel
*          ou Dirichlet tangentiel - Neumann normal
   30 call CDLMIX(tclim,nor,dim,dmeso,phi,cneu)
      call AFFECP(dim,cneu,dim,dmeso,ia,ja,dimanu,anu)
      goto 99
*
*       Transmission
*       Pour l'affectation, le signe n'a d'importance que dans ce cas.
   50 if (nbor.eq.1) then
*       Dirichlet
       call AFFECP(dim,phi,dim,dmeso,ia,ja,dimanu,anu)
*       Neumann
       call AFFECP(dim,cneu,dim,dmeso,it,ja,dimanu,anu)
      else 
*       Dirichlet
       call AFFECM(dim,phi,dim,dmeso,it,ja,dimanu,anu)
*       Neumann
       call AFFECM(dim,cneu,dim,dmeso,ia,ja,dimanu,anu)
      endif
*
   99 return
      end
************************************************************************
      subroutine CDLMIX(tclim,nor,dim,dmeso,phi,cneu)
      integer tclim,dim,dmeso
      real nor(2)
      complex phi(dim,dmeso), cneu(dim,dmeso)
*  Variables locales
      integer j
*       ---------------------------------------------------------------
*       Calcul des conditions aux limites de type mixte sur un bord
*       exterieur de l'ensemble des domaines.
*       Ces conditions sont stockees dans le tableau cneu qui contient
*       en entree les conditions aux limites de type Neumann ou ses
*       derivees par rapport a nu.
*
*       Arguments d'entree  : tclim,nor,dim,dmeso,phi,cneu
*       Arguments de sortie : cneu
*        tclim  : type de condition aux limites sur le bord considere
*        nor    : normale a la frontiere du domaine considere
*        dim    : dimension de l'espace
*        dmeso  : dimension de l'espace des solutions
*        phi    : vecteurs de la base de l'espace des solutions
*                 ou derivee par rapport a nu des vecteurs de base
*                 (phi(.,i), ieme vecteur, i=1,2*dim)
*        cneu   : en entree : stockage des conditions de Neumann ou
*                   derivee par rapport a nu des conditions de Neumann
*                 en sortie : conditions mixtes
*                 (cneu(.,i), ieme vecteur, i=1,2*dim)
*       ---------------------------------------------------------------
      goto (90,90,30,40,90) tclim
*
*       Mixte Dirichlet normal - Neumann tangentiel
*        Calcul des composantes dans la base (n,t)
*        en ecrasant cneu avec circonspection.
*        En dimension 3, (base (n,t,z)) la 3eme composante
*        (tangentielle) est donc deja en place.
   30 do 35 j=1,dmeso
       cneu(2,j) = -cneu(1,j)*nor(2) + cneu(2,j)*nor(1)
       cneu(1,j) =   phi(1,j)*nor(1) +  phi(2,j)*nor(2)
   35 continue
      goto 90
*
*       Mixte Dirichlet tangentiel - Neumann normal
*        Ici, en dimension 3, il faut recuperer la 3eme
*        composante (tangentielle) de phi.
   40 do 45 j=1,dmeso
       cneu(2,j) = cneu(1,j)*nor(1) + cneu(2,j)*nor(2)
       cneu(1,j) = -phi(1,j)*nor(2) +  phi(2,j)*nor(1)
   45 continue
      if (dim.eq.3) then
       do 46 j=1,dmeso
        cneu(3,j) = phi(3,j)
   46  continue
      endif
*
   90 return
      end
************************************************************************
      subroutine CDLNEU(cc,nor,ncb,dim,dmeso,phi,dphidt,dphidn,ddpdtn,
     &  nu,cneu,cneudn)
      integer dim,dmeso
      real cc(*), nor(2), ncb(2)
      complex phi(dim,dmeso), dphidt(dim,dmeso), nu, cneu(dim,dmeso)
      complex dphidn(dim,dmeso), ddpdtn(dim,dmeso),cneudn(dim,dmeso)
*  Points d'entree
      external CDLN2D, CDLN3D
*       ------------------------------------------------------------
*       Calcul des conditions aux limites de type Neumann sur un
*       bord du domaine considere (T = Sigma(u) . n), ceci pour les
*       vecteurs de base de l'espace des solutions.
*       Ce module peut etre utilise pour calculer autre chose que
*       les conditions aux limites ; dans ce cas, la normale n (nor)
*       peut etre choisie arbitrairement.
*       Arguments d'entree  : cc,nor,ncb,dim,dmeso,phi,dphidt,
*                             dphidn,ddpdtn,nu
*       Arguments de sortie : cneu,cneudn
*        cc     : coefficients de la matrice de rigidite du domaine
*        nor    : normale au bord du domaine si on calcule les
*                 conditions aux limite, ou vecteur autre sinon.
*        ncb    : vecteur de changement de base. Doit etre egal a
*                 (-sin(theta), cos(theta)) ou theta est l'angle
*                 polaire definissant le point de calcul.
*        dim    : dimension de l'espace
*        dmeso  : dimension de l'espace des solutions
*        phi    : vecteurs de la base de l'espace des solutions
*                 (phi(.,i), ieme vecteur, i=1,dmeso)
*        dphidt : derivee par rapport a theta des vecteurs de base
*                 (dphidt(.,i), ieme vecteur, i=1,dmeso)
*        dphidn : derivee par rapport a nu des vecteurs de base
*                 (dphidn(.,i), ieme vecteur, i=1,2*dim)
*        ddpdtn : derivee par rapport a theta et a nu des vecteurs
*                 de base (ddpdtn(.,i), ieme vecteur, i=1,2*dim)
*        nu     : exposant de singularite
*        cneu   : conditions de Neumann pour les dmeso vecteurs de
*                 base (cneu(.,i), ieme vecteur, i=1,dmeso)
*        cneudn : derivee par rapport a nu des conditions de Neumann
*                 (cneudn(.,i), ieme vecteur, i=1,2*dim)
*       ------------------------------------------------------------
      if (dim.eq.2) then
        call CDLNMG(cc,nor,ncb,dim,dmeso,phi,dphidt,dphidn,ddpdtn,
     &  nu,cneu,cneudn,CDLN2D)
      else
        call CDLNMG(cc,nor,ncb,dim,dmeso,phi,dphidt,dphidn,ddpdtn,
     &  nu,cneu,cneudn,CDLN3D)
      endif
      end
************************************************************************
      subroutine CDLNMG(cc,nor,ncb,dim,dmeso,phi,dphidt,dphidn,ddpdtn,
     & nu,cneu,cneudn,CCDLN)
      integer dim,dmeso
      real cc(*), nor(2), ncb(2)
      complex phi(dim,dmeso), dphidt(dim,dmeso), nu, cneu(dim,dmeso)
      complex dphidn(dim,dmeso), ddpdtn(dim,dmeso),cneudn(dim,dmeso)
      external CCDLN
*  Variables locales
      integer i,j, ld
      parameter (ld = 3)
      complex d1(ld), d2(ld), temp
*       --------------------------------------------------------------
*       Sous-programme general de calcul des conditions aux limites
*       de type Neumann, ainsi que de leur derivees par rapport a nu,
*       sur un bord du domaine considere en utilisant la routine CCDLN
*       qui caracterise le systeme d'equations.
*
*       Arguments d'entree  : cc,nor,ncb,dim,dmeso,phi,dphidt,
*                             dphidn,ddpdtn,nu
*       Arguments de sortie : cneu,cneudn
*        cc     : coefficients de la matrice de rigidite du domaine
*        nor    : normale au bord du domaine si on calcule les
*                 conditions aux limite, ou vecteur autre sinon.
*        ncb    : vecteur de changement de base. Doit etre egal a
*                 (-sin(theta), cos(theta)) ou theta est l'angle
*                 polaire definissant le point de calcul.
*        dim    : dimension de l'espace
*        dmeso  : dimension de l'espace des solutions
*        phi    : vecteurs de la base de l'espace des solutions
*                 (phi(.,i), ieme vecteur, i=1,dmeso)
*        dphidt : derivee par rapport a theta des vecteurs de base
*                 (dphidt(.,i), ieme vecteur, i=1,dmeso)
*        dphidn : derivee par rapport a nu des vecteurs de base
*                 (dphidn(.,i), ieme vecteur, i=1,2*dim)
*        ddpdtn : derivee par rapport a theta et a nu des vecteurs
*                 de base (ddpdtn(.,i), ieme vecteur, i=1,2*dim)
*        nu     : exposant de singularite
*        cneu   : conditions de Neumann pour les dmeso vecteurs de
*                 base (cneu(.,i), ieme vecteur, i=1,dmeso)
*        cneudn : derivee par rapport a nu des conditions de Neumann
*                 (cneudn(.,i), ieme vecteur, i=1,2*dim)
*       --------------------------------------------------------------
       do 30 j=1,dmeso
        do 10 i=1,dim
         temp = nu*phi(i,j)
         d1(i) =  ncb(2)*temp + ncb(1)*dphidt(i,j)
   10    d2(i) = -ncb(1)*temp + ncb(2)*dphidt(i,j)
        call CCDLN(cc,nor,d1,d2,cneu(1,j))
*
*        Meme chose pour les derivees par rapport a nu
        do 20 i=1,dim
         temp = phi(i,j) + nu*dphidn(i,j)
         d1(i) =  ncb(2)*temp + ncb(1)*ddpdtn(i,j)
   20    d2(i) = -ncb(1)*temp + ncb(2)*ddpdtn(i,j)
        call CCDLN(cc,nor,d1,d2,cneudn(1,j))
   30  continue
      end
************************************************************************
      subroutine CDLN2D(cc,nor,d1,d2,cneu)
      real cc(*), nor(2)
      complex cneu(*), d1(*), d2(*)
*  Variables locales
      complex sdc, coef
*       ----------------------------------------------------------
*       Calcul des conditions aux limites de type Neumann sur un
*       bord du domaine considere (T = Sigma n), en dimension 2.
*
*       Arguments d'entree  : cc,nor,d1,d2
*       Arguments de sortie : cneu
*        cc    : coefficients de la matrice de rigidite du domaine
*        nor   : normale au bord du domaine si on calcule les
*                conditions aux limite, ou vecteur autre sinon.
*        d1,d2 : derivees cartesiennes de la fonction
*        cneu  : conditions de Neumann pour un vecteur de base
*       ----------------------------------------------------------
       sdc = d2(1)+d1(2)
       coef = cc(3)*d1(1)+cc(6)*sdc+cc(5)*d2(2)
       cneu(1) = (cc(1)*d1(1)+cc(3)*sdc+cc(2)*d2(2)) * nor(1) +
     &           coef * nor(2)
       cneu(2) = coef * nor(1) +
     &           (cc(2)*d1(1)+cc(5)*sdc+cc(4)*d2(2)) * nor(2)
      end
************************************************************************
      subroutine CDLN3D(cc,nor,d1,d2,cneu)
      real cc(*), nor(2)
      complex cneu(*), d1(*), d2(*)
*  Variables locales
      complex sdc, coef
*       ----------------------------------------------------------
*       Calcul des conditions aux limites de type Neumann sur un
*       bord du domaine considere (T = Sigma n), en dimension 3.
*
*       Arguments d'entree  : cc,nor,d1,d2
*       Arguments de sortie : cneu
*        cc    : coefficients de la matrice de rigidite du domaine
*        nor   : normale au bord du domaine si on calcule les
*                conditions aux limite, ou vecteur autre sinon.
*        d1,d2 : derivees cartesiennes de la fonction
*        cneu  : conditions de Neumann pour un vecteur de base
*       ----------------------------------------------------------
       sdc = d2(1)+d1(2)
       coef = cc(6)*d1(1) + cc(21)*sdc + cc(20)*d1(3) + cc(11)*d2(2)
     &        + cc(18)*d2(3)
       cneu(1) = (cc(1)*d1(1) + cc(6)*sdc + cc(5)*d1(3) + cc(2)*d2(2)
     &           + cc(4)*d2(3)) * nor(1) + coef * nor(2)
       cneu(2) = coef * nor(1) + (cc(2)*d1(1) + cc(11)*sdc
     &           + cc(10)*d1(3) + cc(7)*d2(2) + cc(9)*d2(3)) * nor(2)
       cneu(3) = (cc(5)*d1(1) + cc(20)*sdc + cc(19)*d1(3) +
     & cc(10)*d2(2)+ cc(17)*d2(3)) * nor(1) + (cc(4)*d1(1) + cc(18)*sdc
     & + cc(17)*d1(3) + cc(9)*d2(2) + cc(16)*d2(3)) * nor(2)
      end
************************************************************************
      subroutine CGMBAS(dim,tab)
      integer dim
      complex tab(dim,*)
*  Variables locales
      integer i,j, jpdim
      complex u, is2
*  Initialisation
      data is2 /(0.,0.5)/
*       ---------------------------------------------------------------
*       Changement de base motive par le fait que si l'exposant de
*       singularite est reel, alors la valeur de la fonction singuliere
*       est aussi reelle.
*       Changement de base (phi(j) et phi(j+dim) sont conjugues) :
*        phi'(j)     = (phi(j) + phi(j+dim)) / 2
*        phi'(j+dim) = (phi(j) - phi(j+dim)) / 2i
*       Arguments d'entree  : dim,tab
*       Arguments de sortie : tab
*        dim : dimension de l'espace
*        tab : tableau de vecteurs ecrits sur la base des phi en entree
*              sur la base des phi' en sortie.
*       ---------------------------------------------------------------
      do 10 i=1,dim
       do 10 j=1,dim
        jpdim = j+dim
        u        = (tab(i,jpdim)-tab(i,j)) * is2
        tab(i,j) = (tab(i,jpdim)+tab(i,j)) * 0.5
   10   tab(i,jpdim) = u
      end
************************************************************************
      subroutine COPRC(matr,matc,nl,nc,sens)
      integer nl, nc, sens
      real    matr(nl,nc)
      complex matc(nl,nc)
*  Variables locales
      integer i, j
*       ----------------------------------------------------
*       Copie (avec conversion de type) de matr (real) dans
*       matc (complex) si sens > 0, de matc dans matr sinon.
*       matr et matc sont des matrices de dimensions (nl,nc).
*        Arguments d'entree  : matr (si sens > 0)
*                              matc (si sens <= 0)
*                              nl,nc,sens
*        Arguments de sortie : matc (si sens > 0)
*                              matr (si sens <= 0)
*       ----------------------------------------------------
      if (sens .gt. 0) then
       do 10 j=1,nc
        do 10 i=1,nl
   10    matc(i,j) = cmplx(matr(i,j))
      else
       do 20 j=1,nc
        do 20 i=1,nl
   20    matr(i,j) = real(matc(i,j))
      endif
*
      end
************************************************************************
      complex function DETCAR(nu)
      complex nu
*  Communs
      include 'singul.inc'
*  Point d'entree
      complex MACDET
*       ---------------------------------------------------------
*       Calcul du determinant caracteristique de la matrice A(nu)
*       Arguments d'entree  : nu
*       Arguments de sortie : valeur de DETCAR
*        nu : exposant de singularite
*       ---------------------------------------------------------
*        Assemblage de la matrice A(nu)
      call ASSANU(cdlim,dmeso,dim,nbcxx,nbdom,dimanu,omega,contx,
     & cc,alpha,qplus,phi,dphidt,cneu,dphidn,ddpdtn,cneudn,nu,anu,anudn)
c MODIF
c       call IMPTAB(' Matrice ANU :',anu,dimanu,dimanu,dimanu)
      DETCAR = MACDET(anu,dimanu,ipvt,work)
      end
************************************************************************
      complex function DPZSDZ(z)
      complex z
*  Communs
      include 'singul.inc'
*  Variables locales
      integer info
*  Points d'entree
      complex TRACEP
*       ----------------------------------------------------------
*       Calcul du terme det'(A(z))/det(A(z)) intervenant dans une
*       integrale de Cauchy.
*       Cette quantite est egale a  trace [A'(z) . inverse(A(z))],
*       ou A'(z) designe la derivee par rapport a z.
*       Si le contour passe trop pres d'un pole, la matrice A(z)
*       est presque singuliere : la fonction renvoie 0 dans ce cas.
*
*       Arguments d'entree  : z
*       Arguments de sortie : valeur de DPZSDZ
*        z : valeur de la variable complexe
*       ----------------------------------------------------------
*
*        Assemblage des matrices A(z)=anu et A'(z)=anudn.
      call ASSANU(cdlim,dmeso,dim,nbcxx,nbdom,dimanu,omega,contx,
     & cc,alpha,qplus,phi,dphidt,cneu,dphidn,ddpdtn,cneudn,z,anu,anudn)
*
*        Calcul de l'inverse de A(z).
      call MACINV(anu,dimanu,info,ipvt,work)
*
*        Si la matrice n'est pas inversible, on renvoie 0 : on annule
*        la contribution de ce point dans l'integrale de Cauchy.
      if (info .ne. 0) then
       DPZSDZ = cmplx(0.)
      else
*
*        Calcul de la trace du produit A'(z) . inverse(A(z))
       DPZSDZ = TRACEP(dimanu,anu,anudn)
      endif
      end
************************************************************************
      subroutine EPCTRL(iu,dim,nbdom,dimker,sseprc,parcou,fact,omega,nu,
     &nomfic,lgfic,codret)
      integer iu,dim,nbdom,dimker,lgfic,codret
      real sseprc,parcou,fact, omega(0:nbdom)
      complex nu
      character*(*) nomfic
*  Variables locales
      integer i
*       -------------------------------------------------------------
*       Ecriture des parametres de controle dans le fichier resultat
*       associe a l'unite logique iu.
*       Arguments d'entree  : iu,dim,nbdom,dimker,sseprc,parcou,fact,
*                             omega,nu,nomfic,lgfic
*        (signification de ces variables dans CALFS)
*       Arguments de sortie : codret
*        codret : code de retour egal a 0 si RAS, #0 sinon (erreur
*                 d'entrees/sorties).
*       -------------------------------------------------------------
*
      codret=0
      write(iu,*,err=60) omega(0), nbdom
      do 10 i=1,nbdom
   10  write(iu,*,err=60) omega(i), nint((omega(i)-omega(i-1))*fact)
      write(iu,50,err=60) real(nu), aimag(nu)
      write(iu,*,err=60) dimker, dim
      write(iu,50,err=60) sseprc, parcou
      write(iu,*,err=60) nomfic(:lgfic)
      return
*
   50 format(g23.15e3,6(1x,g23.15e3))
*
   60 codret=1
      end
************************************************************************
      subroutine EVAFS(dim,phi,coef,val)
      integer dim
      complex phi(dim,*), coef(*), val(*)
*  Variables locales
      integer i,j,jpdim
      complex zero
*  Initialisation
      data zero /(0.,0.)/
*       -------------------------------------------------------------
*       Evaluation de la fonction singuliere en un point.
*       Arguments d'entree  : dim,phi,coef
*       Arguments de sortie : val
*        dim    : dimension de l'espace
*        phi    : vecteurs de la base de l'espace des solutions
*                 (phi(.,i), ieme vecteur, i=1,2*dim)
*        coef   : coefficients de la combinaison lineaire definissant
*                 la fonction singuliere (noyau de A(nu))
*        val    : valeur de la fonction singuliere (vecteur de
*                 dimension dim)
*       -------------------------------------------------------------
      do 10 i=1,dim
   10  val(i) = zero
*
      do 30 j=1,dim
       jpdim = j+dim
       do 20 i=1,dim
   20   val(i) = val(i) + coef(j)*phi(i,j) + coef(jpdim)*phi(i,jpdim)
   30 continue
      end
************************************************************************
      subroutine KERANU(ius,nu,multi,matv,dimanu,s,info,dimker,codret)
      integer ius,multi,dimanu,info,dimker,codret
      complex nu, matv(dimanu,dimanu),s(dimanu)
*  Variables locales
      integer i, j, ordgd, ordgdn, expmin, delord
      parameter (expmin = -16, delord = 3)
      character*52 txtsup
*       ---------------------------------------------------------
*       Determination de la dimension du noyau de A(nu) et
*       impression d'une base de ce noyau.
*       Arguments d'entree  : ius,nu,multi,matv,dimanu,s,info
*       Arguments de sortie : dimker,codret
*        ius    : numero de l'unite logique du fichier ou
*                 sont imprimes les resultats
*        nu     : exposant de singularite
*        multi  : multiplicite de nu
*        matv   : matrice V des vecteurs siguliers a droite
*        dimanu : 1ere dimension des matrices
*        s      : liste des valeurs singulieres classees
*                 par ordre decroissant
*        info   : code de retour de la procedure de decomposition
*                 SVD valant 0 si toutes les valeurs singulieres
*                 sont exactes, > 0 sinon.
*        dimker : dimension du noyau
*        codret : code de retour valant 0 si RAS, 1 si une erreur
*                 est detectee concernant la dimension du noyau.
*       ---------------------------------------------------------
      codret = 0
      write(ius,*)
      write(ius,*) ' +++++++++++++++++'
      write(txtsup,40) nu
      call IMPMES(ius,'KERANU',-1,'-'//txtsup,codret)
      write(txtsup(1:12),'(i12)') multi
      call IMPMES(ius,'KERANU',-2,'-'//txtsup(1:12),codret)
      call IMPMES(ius,'KERANU',-3,' ',codret)
*      Determination de l'ordre de grandeur de la plus petite
*      valeur singuliere, consideree "nulle" puisqu'elle correspond
*      au noyau de A(nu). On imprime le vecteur associe, puis les
*      autres vecteurs engendrant le noyau, si sa dimension est > 1.
      if (s(dimanu) .ne. 0.) then
       ordgdn = nint(log10(real(s(dimanu))))
      else
*        Il arrive qu'une valeur singuliere soit exactement nulle !
       ordgdn = expmin
      endif
      ordgdn = ordgdn + delord
      do 20 j=dimanu,info+1,-1
       if (s(j) .ne. 0.) then
        ordgd = nint(log10(real(s(j))))
       else
        ordgd = ordgdn - 1
       endif
       if (ordgd .gt. ordgdn) then
*       A partir de ce rang, on considere que les valeurs singulieres
*       ne sont plus negligeables.
        goto 30
       else
        write(txtsup(1:2),'(i2)') dimanu-j+1
        call IMPMES(ius,'KERANU',-4,'-'//txtsup(1:2),codret)
        write(txtsup(1:23),'(g23.15e3)') real(s(j))
        call IMPMES(ius,'KERANU',-5,'-'//txtsup(1:23),codret)
        do 10 i=1,dimanu
   10    write(ius,40) matv(i,j)
       endif
   20 continue
      if (info .ne. 0) call IMPMES(ius,'KERANU',6,' ',codret)
      j=0
*
   30 dimker = dimanu-j
      write(ius,*)
      if (dimker .le. multi) then
       write(txtsup(1:2),'(i2)') dimker
       call IMPMES(ius,'KERANU',-7,'-'//txtsup(1:2),codret)
       if (dimker.lt.multi) call IMPMES(ius,'KERANU',-8,' ',codret)
      else
       codret = 1
       write(txtsup(1:24),'(2i12)') multi, dimker
       call IMPMES(ius,'KERANU',9,'-'//txtsup(1:24),codret)
       call IMPMES(  0,'KERANU',9,'-'//txtsup(1:24),codret)
c      write(ius,*) ' La norme 2 de A(nu) vaut ', real(s(1))
c      write(ius,*) ' Ordre grandeur de la + petite v. s. = ', ordgdn-1
      endif
      if (j.ne.0) then
       write(txtsup,40) s(j)
       call IMPMES(ius,'KERANU',-10,'-'//txtsup,codret)
      endif
      write(ius,*) ' +++++++++++++++++'
*
   40 format(1x,g23.15e3,1x,sp,g23.15e3,' *i')
      end
************************************************************************
      complex function MACDET(A,n,ipvt,work)
      integer n, ipvt(n)
      complex A(n,n), work(n)
*  Variables locales
      integer info
      complex det(2)
*      ----------------------------------------------------------
*      Calcule le determinant d'une matrice complexe de rang <= n
*
*       Arguments d'entree  : A,n
*       Arguments de sortie : valeur de MACDET
*       Tableaux de travail : ipvt,work
*      ----------------------------------------------------------
      call cgefa(A,n,n,ipvt,info)
      call cgedi(A,n,n,ipvt,det,work,10)
      MACDET = det(1) * 10.0**det(2)
      end
************************************************************************
      subroutine MACINV(A,n,info,ipvt,work)
      integer n, info, ipvt(n)
      complex A(n,n), work(n)
*  Variables locales
      complex det(2)
*      ------------------------------------------------------------
*      Calcule l'inverse d'une matrice complexe.
*      En sortie, l'inverse occupe la place de la matrice d'entree
*      et la valeur de info est nulle si la matrice est inversible,
*      non nulle sinon.
*
*       Arguments d'entree  : A,n
*       Arguments de sortie : A,info
*       Tableaux de travail : ipvt,work
*      ------------------------------------------------------------
      call cgefa(A,n,n,ipvt,info)
      if (info .eq. 0) call cgedi(A,n,n,ipvt,det,work,1)
      end
************************************************************************
      complex function TRACEP(dimanu,anu,anudn)
      integer dimanu
      complex anu(dimanu,dimanu),anudn(dimanu,dimanu)
*  Variables locales
      integer i, k
      complex t
*       ----------------------------------------------------
*       Calcul de la trace du produit anudn . anu, ou anu
*       et anudn sont des matrices complexes d'ordre dimanu.
*        Arguments d'entree  : dimanu,anu,anudn
*        Arguments de sortie : valeur de TRACEP
*       ----------------------------------------------------
      t = cmplx(0.)
      do 10 i=1,dimanu
       do 10 k=1,dimanu
   10   t = t + anudn(i,k) * anu(k,i)
*
      TRACEP = t
      end
**************************** Fin du fichier ****************************
