************************************************************************
*  Calcul d'exposants de singularites en elasticite anisotrope.
*  Yvon Lafranche, Universite de Rennes 1.
*  Voir documentation a l'adresse :
*        http://perso.univ-rennes1.fr/yvon.lafranche/exsiel/
************************************************************************
************************************************************************
* CMR2G  : construction de la matrice de rigidite en dimension 2
* CMRISO : construction de la matrice de rigidite (cas isotrope)
* LIVDON : lecture, impression et verification des donnees
* LVACT  : lecture, impression et verification de l'action a realiser
* LVANGL : lecture, impression et verification des angles
* LVC2G  : lect, impr et verif des coeff (orthotropie axisymetrique,
*           orthotropie ou elasticite generale, en dimension 2)
* LVC3G  : lect, impr et verif des coefficients (elasticite generale
*           en dimension 3)
* LVCISO : lect, impr et verif des coeff (cas isotrope)
* LVCLIM : lecture, impression et verification des conditions aux limites
* LVCNTX : lecture, impression et verification du contexte de calcul
* LVCOEF : lecture, impression et verification des coefficients
* LVDIM  : lecture, impression et verification de la dimension
* LVDOM  : lecture, impression et verification du nombre de domaines
* LVFRVA : lecture, impression et verification du domaine variable
* LVNUCO : lecture, impr et verif du numero du coefficient Cij variable
* LVRECT : lect, impr et verif du domaine de recherche des racines
* LVRFIB : lect, impr et verif des rotations (orientation des materiaux)
* LVROTA : lect, impr et verif de la rotation (suivi de contour)
* TESPOS : test de positivite de la matrice des coefficients
************************************************************************
      subroutine CMR2G(contx,cc,codret)
      integer contx,codret
      real cc(*)
*       -----------------------------------------------------
*       Construction de la matrice de rigidite en dimension 2
*       par remise en ordre en fonction des coefficients lus.
*       La valeur du contexte doit etre > 1 et < 5 (codret=0)
*       sinon codret = 1.
*       Arguments d'entree  : contx
*       Arguments de sortie : cc,codret
*        contx  : contexte de calcul du domaine considere
*        cc     : coefficients de la matrice de rigidite du
*                 domaine considere
*        codret : code de retour valant 0 si RAS, 1 sinon.
*       -----------------------------------------------------
      codret = 0
      goto (50,20,30,40) contx
   20 cc(6) = cc(3)
      cc(5) = 0.
      cc(4) = cc(1)
      cc(3) = 0.
      goto 40
   30 cc(6) = cc(4)
      cc(5) = 0.
      cc(4) = cc(3)
      cc(3) = 0.
*
   40 return
*
   50 codret = 1
      end
************************************************************************
      subroutine CMRISO(ius,dim,lambda,mu,cc,codret)
      integer ius, dim, codret
      real lambda, mu, cc(*)
*  Variables locales
      integer i
*       -----------------------------------------------------------
*       Construction de la matrice de rigidite dans le cas isotrope
*       a partir des coefficients de Lame lambda et mu :
*       . en dimension 2 :
*        C11 = C22 = lamdba + 2 mu, C12 = lambda,
*        C16 = C26 = 0, C66 = mu
*       . en dimension 3 :
*        C11 = C22 = C33 = lamdba + 2 mu, C12 = C13 = C23 = lambda,
*        C44 = C55 = C66 = mu, les autres nuls.
*       Arguments d'entree  : ius,dim,lambda,mu
*       Arguments de sortie : cc
*        ius    : unite logique du fichier de sortie
*        dim    : dimension de l'espace
*        lambda, mu : coeficients de Lame
*        cc     : coefficients de la matrice de rigidite du
*                 domaine considere
*        codret : code de retour valant 0 si RAS, 1 si les valeurs
*                 lambda et mu sont incompatibles.
*       -----------------------------------------------------------
      codret = 0
*       Test de validite
      if (lambda .le. -mu) then
       call IMPMES(ius,'CMRISO',1,' ',codret)
       call IMPMES(  0,'CMRISO',1,' ',codret)
       codret = 1
       return
      endif
*
      cc(1) = lambda + 2*mu
      cc(2) = lambda
      if (dim .eq. 2) then
       cc(3) = 0.
       cc(4) = cc(1)
       cc(5) = 0.
       cc(6) = mu
      else
       do 10 i=4,20
   10   cc(i) = 0.
       cc(3) = lambda
       cc(7) = cc(1)
       cc(8) = lambda
       cc(12) = cc(1)
       cc(16) = mu   
       cc(19) = mu   
       cc(21) = mu   
      endif 
      end
************************************************************************
      subroutine LIVDON(iue,ius,dimx,nbdomx,nbcxx,prmin,dim,nbdom,contx,
     & cc,omega,cdlim,action,somrec,ang,axe,nudova,nbpado,codret)
      integer iue,ius,dimx,nbdomx,nbcxx,dim,nbdom,contx(nbdom),cdlim(2)
      integer action(2),axe(nbdomx),nudova,nbpado(2),codret
      real prmin, cc(nbcxx,nbdomx), omega(0:nbdomx), somrec(4)
      real ang(3,nbdomx)
*  Variables locales
      integer nbmcx
      parameter (nbmcx = 5)
      logical period
      character*8 motcle(2:nbmcx+1)
*  Initialisation
      data motcle /'ACTION2 ','ACTION3 ','ACTION4 ','ACTION5 ',
     &             'ACTION6 '/
*       ------------------------------------------------------------
*       Lecture, impression et verification des donnees
*       Arguments d'entree  : iue,ius,dimx,nbdomx,nbcxx,prmin
*       Arguments de sortie : dim,nbdom,contx,cc,omega,cdlim,action,
*                             somrec,ang,axe,nudova,codret
*        iue, ius  : unite logique du fichier d'entree, de sortie
*        dimx, dim : dimension max et effective de l'espace
*        nbdomx, nbdom : nombre de domaines max et effectif
*        nbcxx : nombre max de coefficients (1ere dimension de cc)
*        prmin : valeur min de l'abscisse definissant le rectangle
*                pour la recherche automatique de racines
*        cc    : coefficients de la matrice de rigidite pour chaque
*                domaine
*        omega : angles definissant la geometrie des domaines
*        contx : contexte de calcul pour chaque domaine
*                (cf module LVCNTX)
*        cdlim : conditions aux limites en omega(0) et omega(nbdom)
*                (cf module LVCLIM)
*        action : caracterise l'action a realiser
*                (cf module LVACT)
*        somrec : rectangle (xmin,xmax,ymin,ymax) dans lequel
*                 s'effectue la recherche si action(1) > 0
*        ang    : tableau contenant
*                 . dans sa 1ere colonne l'angle de debut, de fin et
*                   l'increment (en radians)
*                   . de la rotation si action(1) = 2
*                   . de la variation angulaire si action(1) = 3
*                 . dans ses nbdom 1ere colonnes l'angle de debut et
*                   l'increment (en radians) si action(1) = 4
*        axe    : numero de l'axe autour duquel se fait la rotation
*        nudova : numero du domaine variable (si action(1) = 3)
*        nbpado : Si action(1) = 4 :
*                 1er elt  = nombre de pas a realiser (identique pour
*                  tous les materiaux)
*                 2eme elt = numero du domaine determinant l'angle a
*                  porter en abscisse des courbes
*                 Si action(1) = 6 :
*                 1er elt  = numero du coefficient Cij a faire varier
*                 2eme elt = numero du domaine concerne
*        codret : code de retour valant 0 si RAS, 1 sinon.
*       ------------------------------------------------------------
      call IMPMES(ius,'LIVDON',-1,' ',codret)
      if (codret.ne.0) goto 10
*
*       Dimension de l'espace
      call LVDIM(iue,ius,dimx,dim,codret)
      if (codret .ne. 0) goto 10
*
*       Nombre de domaines
      call LVDOM(iue,ius,nbdomx,nbdom,codret)
      if (codret .ne. 0) goto 10
*
*       Contexte
      call LVCNTX(iue,ius,dim,nbdom,contx,codret)
      if (codret .ne. 0) goto 10
*
*       Coefficients pour chaque domaine
      call LVCOEF(iue,ius,dim,nbcxx,nbdom,contx,cc,codret)
      if (codret .ne. 0) goto 10
*
*       Angles : omega(0),... omega(nbdom)
      call LVANGL(iue,ius,nbdom,omega,period,codret)
      if (codret .ne. 0) goto 10
*
*       Conditions aux limites
      call LVCLIM(iue,ius,period,cdlim,codret)
      if (codret .ne. 0) goto 10
*
*       Action a realiser
      call LVACT(iue,ius,dim,action(1),action(2),codret)
      if (codret .ne. 0) goto 10
      if (action(1) .eq. 0) return
*
*       Domaine de recherche des racines
      call LVRECT(iue,ius,prmin,somrec,codret)
      if (codret .ne. 0) goto 10
      if (action(1) .eq. 1) return
*
*       Lecture des donnees supplementaires relatives a l'action
*       a realiser (les actions sont mutuellement exclusives)
      if (action(1) .eq. 2) then
*       Definition de la rotation
       call LVROTA(iue,ius,motcle(action(1)),ang,axe,codret)
       if (codret .ne. 0) goto 10
*
      elseif (action(1) .eq. 3) then
*       Determination de la frontiere variable
       call LVFRVA(iue,ius,motcle(action(1)),nbdom,omega,nudova,ang,
     & codret)
       if (codret .ne. 0) goto 10
*
      elseif (action(1) .eq. 4) then
*       Definition des rotations determinant l'orientation des materiaux
       call LVRFIB(iue,ius,motcle(action(1)),nbdom,ang,axe,nbpado,
     & codret)
       if (codret .ne. 0) goto 10
*
      elseif (action(1) .eq. 5) then
*       Suivi d'une arete courbe.
*       Recherche du mot-cle et positionnement au debut des donnees concernees.
       call RCHMCL(iue,motcle(action(1)),codret)
       if (codret .ne. 0) goto 10
*
      elseif (action(1) .eq. 6) then
*       Variation d'un coefficient de la matrice de rigidite.
       call LVNUCO(iue,ius,motcle(action(1)),dim,nbdom,contx,nbpado,
     & codret)
       if (codret .ne. 0) goto 10
      endif
*
      call SEPARR(ius,codret)
      return
*
   10 if (codret.eq.2) then
       call IMPMES(ius,'LIVDON',2,' ',codret)
       call IMPMES(  0,'LIVDON',2,' ',codret)
       codret = 1
      endif
      return
*
      end
************************************************************************
      subroutine LVACT(iue,ius,dim,a0,a1,codret)
      integer iue,ius,dim,a0,a1,codret
*  Variables locales
      logical ok
      real val
      integer action, a0x,a1x, actx, nb, typ, mesref
      parameter (a0x = 6, a1x = 3, actx = 10*a1x + a0x)
      character*12 txtsup
*  Initialisation
      parameter (nb = 1, typ = 1, mesref = 3)
*       ----------------------------------------------------------------
*       Lecture, impression et verification de l'action a realiser.
*       Arguments d'entree  : iue,ius,dim
*       Arguments de sortie : a0,a1,codret
*        iue, ius : unite logique du fichier d'entree, de sortie
*        dim      : dimension de l'espace
*        a0 et a1 caracterisent l'action a realiser :
*          a0 = 0 : Calcul manuel
*               1 : Calcul automatique seul
*               2 : Calcul automatique avec rotation autour d'un
*                   des axes du referenciel absolu
*               3 : Calcul automatique avec variation de la geometrie
*                   dans le referenciel absolu
*               4 : Calcul automatique avec orientation des materiaux
*                   autour d'un des axes du referenciel absolu
*               5 : Calcul automatique avec suivi d'une arete courbe
*               6 : Calcul automatique avec variation d'un coefficient
*                   de la matrice de rigidite
*          a1 = 0 : Noyau non calcule
*               1 : Calcul du noyau de A(nu)
*               2 : 1 + calcul des fonctions singulieres et
*                       des contraintes
*               3 : 2 + tests dans le cas d'une fissure
*        codret : code de retour valant 0 si RAS, #0 sinon.
*       Nota : pour a0 > 0, la recherche des racines se fait dans
*              un rectangle.
*       ----------------------------------------------------------------
      call LECVAL(iue,nb,typ,action,val,codret)
      if (codret .ne. 0) goto 10
      ok = action.ge.0 .and. action.le.actx
      if (ok) then
       a0 = mod(action,10)
       a1 = action / 10
       ok = a0.ge.0 .and. a0.le.a0x .and. a1.ge.0 .and. a1.le.a1x
      endif
      if (.not. ok) then
       write(txtsup,'(i12)') action
       call IMPMES(ius,'LVACT',1,'-'//txtsup,codret)
       call IMPMES(  0,'LVACT',1,'-'//txtsup,codret)
       codret = 1
       goto 10
      endif
*
      call IMPMES(ius,'LVACT',-mesref,' ',codret)
      call ECRVAL(ius,nb,typ,action,val,codret)
      if (codret .ne. 0) goto 10
      action = mesref+1 + a0
      call IMPMES(ius,'LVACT',-action,' ',codret)
C      if (dim.ne.3 .and. (a0.gt.1 .and. a0.ne.3)) then
      if (dim.ne.3 .and. (a0.eq.2 .or. a0.eq.4 .or. a0.eq.5)) then
       call IMPMES(ius,'LVACT',2,' ',codret)
       call IMPMES(  0,'LVACT',2,' ',codret)
       codret = 1
       goto 10
      endif
*
      if (a1.gt.0) then
       action = mesref+2 + a0x + a1
       call IMPMES(ius,'LVACT',-action,' ',codret)
      endif
      call SEPARR(ius,codret)
*
   10 return
      end
************************************************************************
      subroutine LVANGL(iue,ius,nbdom,omega,period,codret)
      integer iue,ius,nbdom,codret
      real omega(0:nbdom)
      logical period
*  Variables locales
      integer i, nb, typ, ival
      real prod, domeg, tcs, epsil, fconv
*  Initialisation
      parameter (typ = 2, tcs = 360., epsil = 1.e-4)
*       ----------------------------------------------------------
*       Lecture, impression et verification des angles definissant
*       la geometrie des domaines.
*       Le domaine i est compris entre omega(i-1) et omega(i).
*       Arguments d'entree  : iue,ius,nbdom
*       Arguments de sortie : omega,period,codret
*        iue, ius : unite logique du fichier d'entree, de sortie
*        nbdom  : nombre de domaines
*        omega  : angles definissant la geometrie des domaines
*        period : indique si les domaines recouvrent tout le plan
*                 Si oui, les conditions aux limites pourront
*                 etre periodiques.
*        codret : code de retour valant 0 si RAS, #0 sinon.
*       ----------------------------------------------------------
      nb = nbdom+1
      call LECVAL(iue,nb,typ,ival,omega(0),codret)
      if (codret .ne. 0) goto 30
      call IMPMES(ius,'LVANGL',-4,' ',codret)
      call ECRVAL(ius,nb,typ,ival,omega,codret)
      if (codret .ne. 0) goto 30
*
      domeg = omega(nbdom)-omega(0)
      period = abs(domeg-tcs).lt.epsil
*      Verification : les angles doivent etre croissants et
*      ne pas recouvrir "plus d'une fois" le plan.
      prod = 1.
      do 10 i=1,nbdom
   10  prod = prod * (omega(i)-omega(i-1))
      if (prod.lt.0. .or. domeg.gt.tcs) then
       call IMPMES(ius,'LVANGL',1,' ',codret)
       call IMPMES(  0,'LVANGL',1,' ',codret)
       if (prod.lt.0.) then
        call IMPMES(ius,'LVANGL',-2,' ',codret)
       else
        call IMPMES(ius,'LVANGL',-3,' ',codret)
       endif
       codret = 1
       goto 30
      endif
*
*       Conversion en radian
      fconv = atan(1.)/45.
      do 20 i=0,nbdom
   20  omega(i) = omega(i)*fconv
      call IMPMES(ius,'LVANGL',-5,' ',codret)
      call ECRVAL(ius,nb,typ,ival,omega,codret)
      if (codret .ne. 0) goto 30
*
      fconv = fconv * 180.
      call IMPMES(ius,'LVANGL',-6,' ',codret)
      write(ius,'(6g13.6)',err=30) (omega(i)/fconv, i=0,nbdom)
      call SEPARR(ius,codret)
*
   30 return
      end
************************************************************************
      subroutine LVC2G(iue,ius,numdom,contx,cc,codret)
      integer iue,ius,numdom,contx,codret
      real cc(*)
*  Variables locales
      integer typ, ival, nb, nbvl(2:4)
      character*24 txtsup
*  Initialisation
      parameter (typ=2)
      data nbvl /3,4,6/
*       -----------------------------------------------------
*       Lecture, impression et verification des coefficients
*       dans les cas : orthotropie axisymetrique, orthotropie
*       ou elasticite generale, en dimension 2, pour le
*       domaine numdom.
*       Arguments d'entree  : iue,ius,numdom,contx
*       Arguments de sortie : cc,codret
*       -----------------------------------------------------
      nb = nbvl(contx)
*       Lecture des nb coefficients
      call LECVAL(iue,nb,typ,ival,cc,codret)
      if (codret .ne. 0) goto 20
      write(txtsup(1:2),'(i2)') numdom
      call IMPMES(ius,'LVC2G',-1,'-'//txtsup(1:2),codret)
      call ECRVAL(ius,nb,typ,ival,cc,codret)
      if (codret .ne. 0) goto 20
*       Construction de la matrice de rigidite
      call CMR2G(contx,cc,codret)
      if (codret .ne. 0) then
       write(txtsup,'(2i12)') contx, numdom
       call IMPMES(ius,'LVC2G',2,'-'//txtsup,codret)
       call IMPMES(  0,'LVC2G',2,'-'//txtsup,codret)
       goto 20
      endif
*
   20 return
      end
************************************************************************
      subroutine LVC3G(iue,ius,numdom,cc,codret)
      integer iue,ius,numdom,codret
      real cc(*)
*  Variables locales
      integer nb, typ, ival
      character*2 txtsup
*  Initialisation
      parameter (nb = 21, typ = 2)
*       ----------------------------------------------------------------
*       Lecture, impression et verification des coefficients dans le cas
*       de l'elasticite generale en dimension 3 pour le domaine numdom.
*       Arguments d'entree  : iue,ius,numdom
*       Arguments de sortie : cc,codret
*       ----------------------------------------------------------------
*       Lecture des nb coefficients
      call LECVAL(iue,nb,typ,ival,cc,codret)
      if (codret .ne. 0) goto 20
      write(txtsup,'(i2)',err=20) numdom
      call IMPMES(ius,'LVC3G',-1,'-'//txtsup,codret)
      call ECRVAL(ius,nb,typ,ival,cc,codret)
      if (codret .ne. 0) goto 20
*
   20 return
      end
************************************************************************
      subroutine LVCISO(iue,ius,dim,numdom,cc,codret)
      integer iue,ius,dim,numdom,codret
      real cc(*)
*  Variables locales
      integer typ, ival, nb
      parameter (nb=2, typ=2)
      real lambda, mu, val(nb)
      character*2 txtsup
      equivalence (val(1), lambda), (val(2), mu)
*       ---------------------------------------------------------
*       Lecture, impression et verification des coefficients
*       dans le cas isotrope, pour le domaine numdom.
*       Arguments d'entree  : iue,ius,dim,numdom
*       Arguments de sortie : cc,codret
*        iue, ius  : unite logique du fichier d'entree, de sortie
*       ---------------------------------------------------------
      call IMPMES(ius,'LVCISO',-1,' ',codret)
*       Lecture des coefficients de Lame lambda et mu
      call LECVAL(iue,nb,typ,ival,val,codret)
      if (codret .ne. 0) goto 20
      write(txtsup,'(i2)') numdom
      call IMPMES(ius,'LVCISO',-2,'-'//txtsup,codret)
      call ECRVAL(ius,nb,typ,ival,val,codret)
      if (codret .ne. 0) goto 20
*       Construction de la matrice de rigidite avec test de validite
      call CMRISO(ius,dim,lambda,mu,cc,codret)
      if (codret .ne. 0) then
       call IMPMES(ius,'LVCISO',3,'-'//txtsup,codret)
       call IMPMES(  0,'LVCISO',3,'-'//txtsup,codret)
       codret = 1
      endif
*
   20 return
      end
************************************************************************
      subroutine LVCLIM(iue,ius,period,cdlim,codret)
      integer iue,ius,cdlim(2),codret
      logical period
*  Variables locales
      integer nbclim, dm, nb, typ
      real val
      integer i, cper
      character*24 txtsup
*  Initialisation
      parameter (nbclim = 5, dm = 3, nb = 2, typ = 1)
*       ---------------------------------------------------------------
*       Lecture, impression et verification des conditions aux limites
*       Arguments d'entree  : iue,ius,period
*       Arguments de sortie : cdlim,codret
*        iue, ius  : unite logique du fichier d'entree, de sortie
*        period : indique si les domaines recouvrent tout le plan
*                 Si oui, les conditions aux limites pourront
*                 etre periodiques.
*        cdlim : conditions aux limites en omega(0) et omega(nbdom)
*                nbdom etant le numero du dernier domaine
*                 1 : Dirichlet
*                 2 : Neumann
*                 3 : Mixte Dirichlet normal - Neumann tangentiel
*                 4 : Mixte Dirichlet tangentiel - Neumann normal
*                 5 : Transmission
*        codret : code de retour valant 0 si RAS, #0 sinon.
*       ---------------------------------------------------------------
*        Initialisation de la condition de transmission (cas periodique)
      cper = nbclim
      call LECVAL(iue,nb,typ,cdlim,val,codret)
      if (codret .ne. 0) goto 20
      call IMPMES(ius,'LVCLIM',-dm,' ',codret)
      call ECRVAL(ius,nb,typ,cdlim,val,codret)
      if (codret .ne. 0) goto 20
      do 10 i=1,nb
       if (cdlim(i).lt.1 .or. cdlim(i).gt.nbclim) then
        write(txtsup(1:12),'(i12)') cdlim(i)
        call IMPMES(ius,'LVCLIM',1,'-'//txtsup(1:12),codret)
        call IMPMES(  0,'LVCLIM',1,'-'//txtsup(1:12),codret)
        codret = 1
        goto 20
       else
        call IMPMES(ius,'LVCLIM',-(dm+cdlim(i)),' ',codret)
       endif
   10 continue
      if ((.not.period) .and. (cdlim(1).eq.cper .or. cdlim(2).eq.cper))
     & then
       write(txtsup,'(2i12)') cdlim
       call IMPMES(ius,'LVCLIM',2,' '//txtsup,codret)
       call IMPMES(  0,'LVCLIM',2,' '//txtsup,codret)
       codret = 1
       goto 20
      endif
      call SEPARR(ius,codret)
*
   20 return
      end
************************************************************************
      subroutine LVCNTX(iue,ius,dim,nbdom,contx,codret)
      integer iue,ius,dim,nbdom,contx(nbdom),codret
*  Variables locales
      integer i,j, nbctxx, nbctx2, nbctx3, dm, typ
      parameter (nbctx2 = 4, nbctx3 = 2)
      parameter (nbctxx = max(nbctx2, nbctx3))
      real val
      integer nbctx(2:3), ctxpos(nbctxx,2:3)
      character*12 txtsup
*  Initialisation
      parameter (dm = 2, typ = 1)
      data nbctx /nbctx2, nbctx3/, ctxpos /1,2,3,4, 1,4,0,0/
*       -----------------------------------------------------------
*       Lecture, impression et verification du contexte de calcul
*       en fonction de la dimension du probleme.
*       Ce parametre depend de la nature physique du materiau dans
*       chaque domaine, et oriente le choix des formules de calcul.
*       Arguments d'entree  : iue,ius,dim,nbdom
*       Arguments de sortie : contx,codret
*        iue, ius : unite logique du fichier d'entree, de sortie
*        dim    : dimension de l'espace
*        nbdom  : nombre de domaines
*        contx  : contexte lu
*                . en dimension 2 :
*                 1 : elasticite isotrope
*                 2 : elasticite orthotrope axisymetrique (C11=C22)
*                 3 : elasticite orthotrope (C16 = C26 = 0)
*                 4 : elasticite generale
*                . en dimension 3 :
*                 1 : elasticite isotrope
*                 4 : elasticite generale
*        codret : code de retour valant 0 si RAS, #0 sinon.
*       -----------------------------------------------------------
      codret = 0
      call LECVAL(iue,nbdom,typ,contx,val,codret)
      if (codret .ne. 0) goto 40
      call IMPMES(ius,'LVCNTX',-dm,' ',codret)
      call ECRVAL(ius,nbdom,typ,contx,val,codret)
      if (codret .ne. 0) goto 40
*
*       Controle et impression de la legende
      do 30 i=1,nbdom
       do 10 j=1,nbctx(dim)
   10   if (contx(i) .eq. ctxpos(j,dim)) goto 20       
       write(txtsup,'(i12)') contx(i)
       call IMPMES(ius,'LVCNTX',1,'-'//txtsup,codret)
       call IMPMES(  0,'LVCNTX',1,'-'//txtsup,codret)
       codret = 1
       goto 40
   20  call IMPMES(ius,'LVCNTX',-(dm+contx(i)),' ',codret)
   30 continue
      call SEPARR(ius,codret)
*
   40 return
      end
************************************************************************
      subroutine LVCOEF(iue,ius,dim,nbcxx,nbdom,contx,cc,codret)
      integer iue,ius,dim,nbcxx,nbdom,contx(nbdom),codret
      real cc(nbcxx,nbdom)
*  Variables locales
      integer i
      logical defpos
      character*2 txtsup
*       ---------------------------------------------------------
*       Lecture, impression et verification des coefficients en
*       fonction de la dimension du probleme et du contexte.
*       Arguments d'entree  : iue,ius,dim,nbcxx,nbdom,contx
*       Arguments de sortie : cc,codret
*        iue, ius : unite logique du fichier d'entree, de sortie
*        dim   : dimension de l'espace
*        contx : contexte de calcul pour chaque domaine
*        nbcxx : nombre max de coefficients (1ere dimension de cc)
*        nbdom : nombre de domaines
*        cc    : coefficients de la matrice de rigidite pour chaque
*                domaine
*        codret : code de retour valant 0 si RAS, #0 sinon.
*       ---------------------------------------------------------
      codret = 0
*       Le nombre de valeurs a lire varie en fonction de la dimension
*       du probleme et du contexte. La construction de la matrice de
*       rigidite en depend egalement.
      do 30 i=1,nbdom
       if (dim.eq.2) then
*         Dimension 2
        if (contx(i) .eq. 1) then
         call LVCISO(iue,ius,dim,i,cc(1,i),codret)
        else
         call LVC2G(iue,ius,i,contx(i),cc(1,i),codret)
        endif
       else
*         Dimension 3
        if (contx(i) .eq. 1) then
         call LVCISO(iue,ius,dim,i,cc(1,i),codret)
        else
         call LVC3G(iue,ius,i,cc(1,i),codret)
        endif
       endif
   30 continue       
*
      if (codret .ne. 0) goto 60
      call IMPMES(ius,'LVCOEF',-1,' ',codret)
      do 50 i=1,nbdom
       write(txtsup,'(i2)') i
       call IMPMES(ius,'LVCOEF',-2,'-'//txtsup,codret)
       call ECRCOF(ius,dim,cc(1,i),codret)
       if (codret .ne. 0) goto 60
*       La matrice doit etre definie positive
       call TESPOS(dim,cc(1,i),defpos,codret)
       if (.not. defpos) then
        call IMPMES(ius,'LVCOEF',3,'-'//txtsup,codret)
        call IMPMES(  0,'LVCOEF',3,'-'//txtsup,codret)
        codret = 1
        goto 60
       endif
   50 continue
*
      call SEPARR(ius,codret)
*
   60 return
      end
************************************************************************
      subroutine LVDIM(iue,ius,dimx,dim,codret)
      integer iue,ius,dimx,dim,codret
*  Variables locales
      real val
      integer nb, typ
      character*12 txtsup
*  Initialisation
      parameter (nb = 1, typ = 1)
*       ---------------------------------------------------------
*       Lecture, impression et verification de la dimension
*       Arguments d'entree  : iue,ius,dimx
*       Arguments de sortie : dim,codret
*        iue, ius : unite logique du fichier d'entree, de sortie
*        dimx : dimension max admissible
*        dim  : dimension lue
*        codret : code de retour valant 0 si RAS, #0 sinon.
*       ---------------------------------------------------------
      call LECVAL(iue,nb,typ,dim,val,codret)
      if (codret .ne. 0) goto 10
      if (dim.lt.2 .or. dim.gt.dimx) then
       write(txtsup,'(i12)') dim
       call IMPMES(ius,'LVDIM',1,'-'//txtsup,codret)
       call IMPMES(  0,'LVDIM',1,'-'//txtsup,codret)
       codret = 1
       goto 10
      endif
      call IMPMES(ius,'LVDIM',-2,' ',codret)
      call ECRVAL(ius,nb,typ,dim,val,codret)
      if (codret .ne. 0) goto 10
      call SEPARR(ius,codret)
*
   10 return
      end
************************************************************************
      subroutine LVDOM(iue,ius,nbdomx,nbdom,codret)
      integer iue,ius,nbdomx,nbdom,codret
*  Variables locales
      real val
      integer nb, typ
      character*24 txtsup
*  Initialisation
      parameter (nb = 1, typ = 1)
*       ---------------------------------------------------------
*       Lecture, impression et verification du nombre de domaines
*       Arguments d'entree  : iue,ius,nbdomx
*       Arguments de sortie : nbdom,codret
*        iue, ius : unite logique du fichier d'entree, de sortie
*        nbdomx : nombre de domaines max admissible
*        nbdom  : nombre de domaines lu
*        codret : code de retour valant 0 si RAS, #0 sinon.
*       ---------------------------------------------------------
      call LECVAL(iue,nb,typ,nbdom,val,codret)
      if (codret .ne. 0) goto 10
      if (nbdom.lt.1 .or.  nbdom.gt.nbdomx) then
        write(txtsup,'(2i12)') nbdom, nbdomx
       call IMPMES(ius,'LVDOM',1,'-'//txtsup,codret)
       call IMPMES(  0,'LVDOM',1,'-'//txtsup,codret)
       codret = 1
       goto 10
      endif
      call IMPMES(ius,'LVDOM',-2,' ',codret)
      call ECRVAL(ius,nb,typ,nbdom,val,codret)
      if (codret .ne. 0) goto 10
      call SEPARR(ius,codret)
*
   10 return
      end
************************************************************************
      subroutine LVFRVA(iue,ius,motcle,nbdom,omega,nufron,ang,codret)
      integer iue,ius,nbdom,nufron,codret
      character*(*) motcle
      real omega(0:nbdom), ang(3)
*  Variables locales
      integer i, nb, typ, ival
      real val, fconv, dpi, angmin, angmax
      character*42 txtsup
*  Initialisation
      parameter (dpi = 6.28318530717958647688)
*       ----------------------------------------------------------------
*       Lecture, impression et verification de la frontiere variable.
*       Si la frontiere est interne (0 < nufron < nbdom), le changement
*       de position de la frontiere entraine la deformation des deux
*       domaines adjacents ; elle ne concerne qu'un seul domaine si la
*       frontiere est celle d'un bord (nufron = 0 ou nufron = nbdom) et
*       la variation est limitee respectivement par l'angle omega(nbdom)
*       ou omega(0).
*       Dans le cas periodique, la frontiere variable doit etre interne,
*       ce qui est toujours possible quite a translater l'origine des
*       angles omega(0).
*       Arguments d'entree  : iue,ius,motcle
*       Arguments de sortie : nufron,ang,codret
*        iue, ius : unite logique du fichier d'entree, de sortie
*        motcle : mot cle caracterisant l'action a realiser
*        nbdom  : nombre de domaines
*        omega  : angles definissant la geometrie des domaines (en rd)
*        nufron : numero de la frontiere variable
*        ang    : tableau contenant l'angle de debut, de fin et
*                 l'increment de la variation angulaire (en radians)
*        codret : code de retour valant 0 si RAS, #0 sinon.
*       ----------------------------------------------------------------
      call RCHMCL(iue,motcle,codret)
      if (codret .ne. 0) goto 20
      call IMPMES(ius,'LVFRVA',-1,' ',codret)
      if (codret .ne. 0) goto 20
*
      nb = 1
      typ = 1
      call LECVAL(iue,nb,typ,nufron,val,codret)
      if (codret .ne. 0) goto 20
      if (nufron.lt.0 .or. nufron.gt.nbdom) then
       write(txtsup(1:12),'(i12)') nufron
       call IMPMES(ius,'LVFRVA',2,'-'//txtsup(1:12),codret)
       call IMPMES(  0,'LVFRVA',2,'-'//txtsup(1:12),codret)
       codret = 1
       goto 20
      endif
      call IMPMES(ius,'LVFRVA',-4,' ',codret)
      call ECRVAL(ius,nb,typ,nufron,val,codret)
      if (codret .ne. 0) goto 20
      nb = 3
      typ = 2
      call LECVAL(iue,nb,typ,ival,ang,codret)
      if (codret .ne. 0) goto 20
      call IMPMES(ius,'LVFRVA',-5,' ',codret)
      call ECRVAL(ius,nb,typ,ival,ang,codret)
      if (codret .ne. 0) goto 20
*      Conversion en radian
      fconv = atan(1.)/45.
      do 10 i=1,nb
   10  ang(i) = ang(i)*fconv
*
      if (nufron .eq. 0) then
       angmin = omega(nbdom) - dpi
       angmax = omega(nufron+1)
      elseif (nufron .eq. nbdom) then
       angmin = omega(nufron-1)
       angmax = omega(0) + dpi
      else
       angmin = omega(nufron-1)
       angmax = omega(nufron+1)
      endif
      if ((ang(2)-ang(1))*ang(3).lt.0.
     &    .or. ang(1).lt.angmin .or. ang(2).gt.angmax) then
       write(txtsup,'(3(1x,g13.6))') ang
       call IMPMES(ius,'LVFRVA',3,' '//txtsup,codret)
       call IMPMES(  0,'LVFRVA',3,' '//txtsup,codret)
       codret = 1
       goto 20
      endif
*
      call IMPMES(ius,'LVFRVA',-6,' ',codret)
      call ECRVAL(ius,nb,typ,ival,ang,codret)
      if (codret .ne. 0) goto 20
      call SEPARR(ius,codret)
*
   20 return
      end
************************************************************************
      subroutine LVNUCO(iue,ius,motcle,dim,nbdom,contx,nucodo,codret)
      integer iue,ius,dim,nbdom,contx(nbdom),nucodo(2),codret
      character*(*) motcle
*  Variables locales
      integer i, nb, dm, typ, nb2d, nb2dp1, nb3d, nbx, nucoef, ctx
      parameter (nb2d=6, nb2dp1=nb2d+1, nb3d=21, nbx=nb2d+nb3d)
      integer coef(nbx), ideb(2:3), ifin(2:3), valaut(0:4,3)
      real val
      character*12 txtsup
*  Initialisation
      parameter (dm = 4)
      data coef /11, 12, 16, 22, 26, 66,
     &           11, 12, 13, 14, 15, 16,
     &               22, 23, 24, 25, 26,
     &                   33, 34, 35, 36,
     &                       44, 45, 46,
     &                           55, 56,
     &                               66/
      data ideb /1,nb2dp1/, ifin /nb2d, nbx/
      data valaut /
     & 2, 12, 66,  0,  0,
     & 3, 11, 12, 66,  0,
     & 4, 11, 12, 22, 66/
*       --------------------------------------------------------------
*       Lecture, impression et verification du numero du coefficient
*       Cij de la matrice de rigidite a faire varier.
*       Arguments d'entree  : iue,ius,motcle,dim
*       Arguments de sortie : nucodo,codret
*        iue, ius : unite logique du fichier d'entree, de sortie
*        motcle : mot cle caracterisant l'action a realiser
*        dim    : dimension de l'espace
*        nbdom  : nombre de domaines
*        contx  : contexte de calcul pour chaque domaine
*        nucodo : 1er elt  = indice dans le tableau commun (cc) du
*                 coefficient Cij destine a varier.
*                 Vaut 0 en sortie en cas d'erreur.
*                 2eme elt = numero du domaine ou les caracteristiques
*                 du materiau sont destinees a varier
*        codret : code de retour valant 0 si RAS, #0 sinon.
*       --------------------------------------------------------------
      call RCHMCL(iue,motcle,codret)
      if (codret .ne. 0) goto 50
      call IMPMES(ius,'LVNUCO',-1,' ',codret)
      if (codret .ne. 0) goto 50
*
      nb = 2
      typ = 1
      call LECVAL(iue,nb,typ,nucodo,val,codret)
      if (codret .ne. 0) goto 50
      call ECRVAL(ius,nb,typ,nucodo,val,codret)
      if (nucodo(2).lt.1 .or. nucodo(2).gt.nbdom) then
       write(txtsup,'(i12)') nucodo(2)
       call IMPMES(ius,'LVNUCO',2,'-'//txtsup,codret)
       call IMPMES(  0,'LVNUCO',2,'-'//txtsup,codret)
       codret = 1
       goto 50
      endif
*
      nucoef = nucodo(1)
      ctx = contx(nucodo(2))
*
*      Controle du numero du coefficient en fonction du contexte
      if (ctx .le. 3) then
       do 10 i=1,valaut(0,ctx)
        if (nucoef .eq. valaut(i,ctx)) goto 20
   10  continue
*       Le coefficient choisi ne peut pas varier
*       dans le contexte courant.
       goto 40
      endif
*
*      Determination de l'indice interne (et controle)
   20 do 30 i=ideb(dim),ifin(dim)
       if (nucoef .eq. coef(i)) then
        nucodo(1) = i - ideb(dim)+1
        write(txtsup,'(i12)') nucodo(1)
        call IMPMES(ius,'LVNUCO',-dm,'-'//txtsup,codret)
        write(txtsup,'(i12)') nucodo(2)
        call IMPMES(ius,'LVNUCO',-(dm+1),'-'//txtsup,codret)
        return
       endif
   30 continue
   40 write(txtsup,'(i12)') nucoef
      call IMPMES(ius,'LVNUCO',3,'-'//txtsup,codret)
      call IMPMES(  0,'LVNUCO',3,'-'//txtsup,codret)
      nucodo(1) = 0
      codret = 1
   50 end
************************************************************************
      subroutine LVRECT(iue,ius,prmin,somrec,codret)
      integer iue,ius,codret
      real prmin, somrec(4)
*  Variables locales
      integer nb, typ, ival
*  Initialisation
      parameter (nb = 4, typ = 2)
*       ------------------------------------------------------------
*       Lecture, impression et verification du domaine rectangulaire
*       de recherche des racines.
*       Arguments d'entree  : iue,ius,prmin
*       Arguments de sortie : somrec,codret
*        iue, ius  : unite logique du fichier d'entree, de sortie
*        prmin  : valeur min de l'abscisse definissant le rectangle
*        somrec : rectangle (xmin,xmax,ymin,ymax) dans lequel
*                 s'effectue la recherche
*        codret : code de retour valant 0 si RAS, #0 sinon.
*       ------------------------------------------------------------
      call LECVAL(iue,nb,typ,ival,somrec,codret)
      if (codret .ne. 0) goto 10
      call IMPMES(ius,'LVRECT',-1,' ',codret)
      call ECRVAL(ius,nb,typ,ival,somrec,codret)
      if (codret .ne. 0) goto 10
*       Verification de la coherence
      if (somrec(1).lt.prmin .or. somrec(2).le.somrec(1) .or.
     & somrec(4).le.somrec(3)) then
       call IMPMES(ius,'LVRECT',2,' ',codret)
       call IMPMES(  0,'LVRECT',2,' ',codret)
       codret = 1
       goto 10
      endif
      call SEPARR(ius,codret)
*
   10 return
      end
************************************************************************
      subroutine LVRFIB(iue,ius,motcle,nbdom,ang,axe,nbpado,codret)
      integer iue,ius,nbdom,axe(nbdom),nbpado(2),codret
      character*(*) motcle
      real ang(3,nbdom)
*  Variables locales
      integer i,j, nbtot, nbr, nbi, typr, typi, ival
      real val, tcs, fconv
      character*2 txtsup
*  Initialisation
      parameter (tcs=360., nbtot=3, nbr=2,nbi=nbtot-nbr, typr=2,typi=1)
*       ---------------------------------------------------------------
*       Lecture, impression et verification des caracteristiques (angle
*       et axe) des rotations determinant l'orientation des fibres des
*       materiaux dans le repere absolu.
*       Arguments d'entree  : iue,ius,motcle,nbdom
*       Arguments de sortie : ang,axe,nbpado,codret
*        iue, ius : unite logique du fichier d'entree, de sortie
*        motcle : mot cle caracterisant l'action a realiser
*        nbdom  : nombre de domaines
*        ang    : tableau contenant l'angle de debut et l'increment de
*                 la rotation (en radians) pour chaque domaine
*        axe    : pour chaque domaine, numero de l'axe autour duquel
*                 se fait la rotation
*        nbpado : 1er elt  = nombre de pas a realiser (identique pour
*                  tous les materiaux)
*                 2eme elt = numero du domaine determinant l'angle a
*                  porter en abscisse des courbes
*        codret : code de retour valant 0 si RAS, #0 sinon.
*       ---------------------------------------------------------------
      call RCHMCL(iue,motcle,codret)
      if (codret .ne. 0) goto 30
      call IMPMES(ius,'LVRFIB',-1,' ',codret)
      if (codret .ne. 0) goto 30
      fconv = atan(1.)/45.
*
      do 20 j=1,nbdom
       call LECVAL(iue,nbtot,typr,ival,ang(1,j),codret)
       if (codret .ne. 0) goto 30
       axe(j) = nint(ang(3,j))
       write(txtsup,'(i2)') j
       call IMPMES(ius,'LVRFIB',-2,'-'//txtsup,codret)
       call IMPMES(ius,'LVRFIB',-3,' ',codret)
       call ECRVAL(ius,nbr,typr,ival,ang(1,j),codret)
       if (codret .ne. 0) goto 30
*       Conversion en radian
       do 10 i=1,nbr
   10   ang(i,j) = ang(i,j)*fconv
       call IMPMES(ius,'LVRFIB',-4,' ',codret)
       call ECRVAL(ius,nbr,typr,ival,ang(1,j),codret)
       if (codret .ne. 0) goto 30
       call IMPMES(ius,'LVRFIB',-5,' ',codret)
       call ECRVAL(ius,nbi,typi,axe(j),val,codret)
       if (codret .ne. 0) goto 30
       if (abs(ang(1,j)).gt.tcs .or. axe(j).lt.1 .or. axe(j).gt.3) then
        call IMPMES(ius,'LVRFIB',6,' ',codret)
        call IMPMES(  0,'LVRFIB',6,' ',codret)
        codret = 1
        goto 30
       endif
   20  continue
*
*      Lecture du nombre de pas et du numero de domaine de reference
*      pour les graphiques
      call LECVAL(iue,nbr,typi,nbpado,val,codret)
      if (codret .ne. 0) goto 30
      call IMPMES(ius,'LVRFIB',-7,' ',codret)
      call ECRVAL(ius,nbr,typi,nbpado,val,codret)
      if (codret .ne. 0) goto 30
      if (nbpado(1).lt.0 .or. nbpado(2).lt.1.or.nbpado(2).gt.nbdom) then
       call IMPMES(ius,'LVRFIB',8,' ',codret)
       call IMPMES(  0,'LVRFIB',8,' ',codret)
      endif
      call SEPARR(ius,codret)
*
   30 return
      end
************************************************************************
      subroutine LVROTA(iue,ius,motcle,ang,axe,codret)
      integer iue,ius,axe,codret
      character*(*) motcle
      real ang(3)
*  Variables locales
      integer i, nb, typ, ival
      real val, fconv
*       ---------------------------------------------------------------
*       Lecture, impression et verification des caracteristiques (angle
*       et axe) de la rotation correspondant au contour a suivre.
*       Arguments d'entree  : iue,ius,motcle
*       Arguments de sortie : ang,axe,codret
*        iue, ius : unite logique du fichier d'entree, de sortie
*        motcle : mot cle caracterisant l'action a realiser
*        ang    : tableau contenant l'angle de debut, de fin et
*                 l'increment de la rotation (en radians)
*        axe    : numero de l'axe autour duquel se fait la rotation
*        codret : code de retour valant 0 si RAS, #0 sinon.
*       ---------------------------------------------------------------
      call RCHMCL(iue,motcle,codret)
      if (codret .ne. 0) goto 20
      call IMPMES(ius,'LVROTA',-1,' ',codret)
      if (codret .ne. 0) goto 20
*
      nb = 3
      typ = 2
      call LECVAL(iue,nb,typ,ival,ang,codret)
      if (codret .ne. 0) goto 20
      call IMPMES(ius,'LVROTA',-3,' ',codret)
      call ECRVAL(ius,nb,typ,ival,ang,codret)
      if (codret .ne. 0) goto 20
      if ((ang(2)-ang(1))*ang(3) .lt. 0.) then
       call IMPMES(ius,'LVROTA',2,' ',codret)
       call IMPMES(  0,'LVROTA',2,' ',codret)
       codret = 1
       goto 20
      endif
*      Conversion en radian
      fconv = atan(1.)/45.
      do 10 i=1,nb
   10  ang(i) = ang(i)*fconv
      call IMPMES(ius,'LVROTA',-4,' ',codret)
      call ECRVAL(ius,nb,typ,ival,ang,codret)
      if (codret .ne. 0) goto 20
      nb = 1
      typ = 1
      call LECVAL(iue,nb,typ,axe,val,codret)
      if (codret .ne. 0) goto 20
      call IMPMES(ius,'LVROTA',-5,' ',codret)
      call ECRVAL(ius,nb,typ,axe,val,codret)
      if (codret .ne. 0) goto 20
      if (axe.lt.1 .or. axe.gt.3) then
       call IMPMES(ius,'LVROTA',6,' ',codret)
       call IMPMES(  0,'LVROTA',6,' ',codret)
       codret = 1
       goto 20
      endif
      call SEPARR(ius,codret)
*
   20 return
      end
************************************************************************
      subroutine TESPOS(dim,cc,defpos,codret)
      integer dim, codret
      real cc(*)
      logical defpos
*  Variables locales
      integer nlx
      parameter (nlx = 6)
      real A(nlx,nlx), eps
      integer i,j,k, nordre, ordpos(2:3), retcho
*  Initialisation
      data ordpos /3, 6/, eps /1.e-4/
*       -------------------------------------------------------------
*       Teste si la matrice des coefficients est definie positive.
*       Pour cela, on calcule sa decomposition de Cholesky.
*       Arguments d'entree  : nb,cc
*       Arguments de sortie : defpos,codret
*        dim    : dimension de l'espace
*        cc     : coefficients de la matrice
*        defpos : vrai si la matrice est definie positive, faux sinon
*        codret : code de retour valant 0 si RAS, 1 si la dimension
*                 de l'espace est incorrecte.
*       -------------------------------------------------------------
      codret = 0
      if (dim.ne.2 .and. dim.ne.3) goto 20
      nordre = ordpos(dim)
*       Construction de la matrice A
      k = 0
      do 10 j=1,nordre
       do 10 i=j,nordre
        k = k+1
   10   A(i,j) = cc(k)
*       Decomposition de Cholesky de la matrice A
      call DECHOL(A,nlx,nordre,A,nlx,eps,retcho)
      defpos = retcho.eq.0
      return
*
   20 codret = 1
      end
**************************** Fin du fichier ****************************
