************************************************************************
*  Calcul d'exposants de singularites en elasticite anisotrope.
*  Yvon Lafranche, Universite de Rennes 1.
*  Voir documentation a l'adresse :
*        http://perso.univ-rennes1.fr/yvon.lafranche/exsiel/
************************************************************************
************** Debut du fichier d'inclusion "singul.inc" ***************
*
*       Definition des parametres relatifs a la recherche des racines
*       -------------------------------------------------------------
*
* Terminologie : sans autre precision, le mot "racine" designe
*                generalement un exposant de singularite.
*
* nbpcer,nbprec : nombre de points pour le calcul des integrales 
*                 de contour sur un cercle et sur un rectangle
      integer nbpcer,nbprec
      parameter (nbpcer = 64, nbprec = 10)
* prmin  : valeur minimum de l'abscisse definissant le rectangle
*          pour la recherche automatique de racines.
*          On ne s'interesse qu'aux racines non entieres de partie
*          reelle positive, d'ou la valeur choisie.
* pimin  : partie imaginaire min en deca de laquelle les exposants ne
*          sont pas recherches, du fait que si un nombre complexe est
*          racine, alors son conjugue l'est aussi.
*          En pratique, on utilise -sseprc (voir ci-dessous), mais
*          cette valeur n'est pas obligatoire.
* taimai : taille min de la maille rectangulaire lors du
*          decoupage du plan pour la recherche des racines
* toler  : precision requise pour le calcul des exposants
*          La valeur fixee ci-dessous est prevue pour des calculs en double
*          precision car la simple precision n'est pas suffisante dans beaucoup
*          de cas. Si la simple precision est requise, prendre toler de l'ordre
*          de 5.e-5 sans toucher aux autres parametres.
      real prmin, pimin, taimai, toler
      parameter (toler = 1.e-9, taimai = 5.e-5)
      parameter (prmin = toler, pimin = -2*toler)
*
*       Autres parametres
*       -----------------
* sseprc  : seuil de separation des racines reelles et complexes
      real sseprc
      parameter (sseprc = 2*toler)
************************************************************************
*
*       Definition des parametres de dimensionnement
*       --------------------------------------------
* dimx   : dimension max de l'espace dans lequel le probleme est pose
* dmesox : dimension max de l'espace des solutions
* nbdomx : nombre max de domaines a prendre en compte
* nbcxx  : nombre de coefficients de la matrice de rigidite a stoker
* nbintx : nombre max d'intervalles de la discretisation utilisee pour
*          le calcul de la fonction d'analyse
      integer dimx,dmesox,nbdomx,nbcxx,nbintx
      parameter (dimx = 3, dmesox = 2*dimx, nbdomx = 4, nbcxx = 21)
      parameter (nbintx = 80)
* dim    : dimension effective de l'espace
* nbdom  : nombre effectif de domaines
* dmeso  : dimension de l'espace des solutions
* dimanu : dimension effective de la matrice anu
      integer dim,nbdom,dmeso,dimanu
*
*       Declaration des tableaux
*       ------------------------
* cdlim  : type des conditions aux limites
* contx  : contexte du calcul en fonction de la nature des materiaux dans
*          chaque domaine (isotrope, orthotrope, anisotrope)
      integer cdlim(2), contx(nbdomx)
* cc     : coefficients des matrices de rigidite 3D dans chaque domaine.
*          Une telle matrice C etant symetrique, on ne stocke (sous
*          forme de vecteur) que la partie triangulaire superieure,
*          ligne par ligne :
*                 C11 C12 C13 C14 C15 C16 ( 6)
*                     C22 C23 C24 C25 C26 (11)
*                         C33 C34 C35 C36 (15)
*                             C44 C45 C46 (18)
*                                 C55 C56 (20)
*                                     C66 (21)
*         Cij est stoke dans cc(k,.) ou k = 1 + (14-i)*(i-1)/2 + j-i.
*         En 2D, seuls sont utilises cc(k,.), k = 1,6 corespondant
*         respectivement a C11, C12, C16, C22, C26, C66.
      real cc(nbcxx,nbdomx)
* omega  : angles definissant la geometrie des domaines a partir de l'angle
*          de reference omega(0). Le domaine No j est defini par le secteur
*          angulaire omega(j-1) - omega(j), j dans [1,nbdom].
      real omega(0:nbdomx)
* alpha  : racines du determinant du symbole M+(alpha)
*          alpha(i,j) = ieme racine sur le domaine No j, i dans [1,dim].
      complex alpha(dimx,nbdomx)
* qplus  : colonne non nulle de M+ad(alphai)
*          qplus(.,i,j) = vecteur d'ordre dim correspondant a alpha(i,j)
      complex qplus (dimx,dimx,nbdomx)
*
*  ---------------- Les tableaux suivants sont modifies  ----------------
*  ---------------- systematiquement au cours du calcul. ----------------
* phi    : base de l'espace des solutions pour un domaine
* dphidt : derivee par rapport a theta des composantes des vecteurs
*          de base precedents
* cneu   : condition de Neumann pour les vecteurs de base
      complex phi(dimx,dmesox), dphidt(dimx,dmesox), cneu(dimx,dmesox)

* dphidn : derivee par rapport a nu des composantes des vecteurs de base
* ddpdtn : derivee par rapport a theta et a nu des composantes des
*          vecteurs de base
* cneudn : derivee par rapport a nu des conditions de Neumann pour les
*          vecteurs de base
      complex dphidn(dimx,dmesox), ddpdtn(dimx,dmesox)
      complex cneudn(dimx,dmesox)
* anu    : matrice A(nu) pour laquelle on cherche nu tq det(A(nu)) = 0,
*          nu exposant de singularite.
* anudn  : derivee de la matrice anu par rapport a nu
      complex anu(dmesox*nbdomx,dmesox*nbdomx)
      complex anudn(dmesox*nbdomx,dmesox*nbdomx)
*
* Pour le calcul des racines :
      real absci(nbintx)
* Pour les sous-programmes LINPAC :
      integer ipvt(dmesox*nbdomx)
      complex work(dmesox*nbdomx)
      real   rwork(dmesox*nbdomx)
      EQUIVALENCE (work, rwork)
* ranu : matrice reelle occupant le meme emplacement que anudn
      real ranu(dmesox*nbdomx,dmesox*nbdomx)
      EQUIVALENCE (ranu, anudn)
************************************************************************
*
*       Definition du commun
*       --------------------
*      1) Type complex
      common alpha,anu,anudn, cneu,cneudn, ddpdtn,dphidt,dphidn
      common phi, qplus, work
*      2) Type real
      common absci, cc, omega
*      3) Type integer
      common cdlim,contx,dim,nbdom,dmeso,dimanu, ipvt
**************************** Fin du fichier ****************************
